@@ -76,7 +76,7 @@ public class GitMoveDeleteHookTest {
 		SystemReader.setInstance(mockSystemReader);
 		mockSystemReader.setProperty(Constants.GIT_CEILING_DIRECTORIES_KEY,
 				ResourcesPlugin.getWorkspace().getRoot().getLocation().toFile()
-						.getAbsoluteFile().toString());
+						.getParentFile().getAbsoluteFile().toString());
 		workspaceSupplement = testUtils.createTempDir("wssupplement");
 		testDirs.add(testUtils.getBaseTempDir());
 		workspace = ResourcesPlugin.getWorkspace().getRoot().getLocation().toFile().getAbsoluteFile();
@@ -91,6 +91,7 @@ public class GitMoveDeleteHookTest {
 		for (File d : testDirs)
 			if (d.exists())
 				FileUtils.delete(d, FileUtils.RECURSIVE | FileUtils.RETRY);
+		SystemReader.setInstance(null);
 	}
 
 	private TestProject initRepoInsideProjectInsideWorkspace() throws IOException,
