@@ -13,6 +13,7 @@ import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
@@ -27,6 +28,7 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
+import org.eclipse.egit.core.internal.CoreText;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.FileMode;
@@ -281,7 +283,7 @@ public class ContainerTreeIterator extends WorkingTreeIterator {
 		final IResource rsrc;
 		final boolean hasInheritedResourceFilters;
 
-		private final FileMode mode;
+		private final FileMode fileMode;
 
 		private long length = -1;
 
@@ -292,7 +294,9 @@ public class ContainerTreeIterator extends WorkingTreeIterator {
 			FileMode mode = null;
 			try {
 				File file = asFile();
-				if (FS.DETECTED.supportsSymlinks() && file != null
+				if (file == null)
+					mode = FileMode.MISSING;
+				else if (FS.DETECTED.supportsSymlinks()
 						&& FS.DETECTED.isSymLink(file))
 					mode = FileMode.SYMLINK;
 				else {
@@ -321,12 +325,12 @@ public class ContainerTreeIterator extends WorkingTreeIterator {
 			} catch (IOException e) {
 				mode = FileMode.MISSING;
 			}
-			this.mode = mode;
+			this.fileMode = mode;
 		}
 
 		@Override
 		public FileMode getMode() {
-			return mode;
+			return fileMode;
 		}
 
 		@Override
@@ -340,11 +344,12 @@ public class ContainerTreeIterator extends WorkingTreeIterator {
 		@Override
 		public long getLength() {
 			if (length < 0)
-				if (rsrc instanceof IFile || mode == FileMode.SYMLINK) {
+				if (rsrc instanceof IFile || fileMode == FileMode.SYMLINK
+						|| fileMode == FileMode.GITLINK) {
 					try {
 						File file = asFile();
 						if (file != null)
-							length = FS.DETECTED.length(asFile());
+							length = FS.DETECTED.length(file);
 						else
 							length = 0;
 					} catch (IOException e) {
@@ -357,9 +362,12 @@ public class ContainerTreeIterator extends WorkingTreeIterator {
 
 		@Override
 		public long getLastModified() {
-			if (mode == FileMode.SYMLINK) {
+			if (fileMode == FileMode.SYMLINK) {
 				try {
-					return FS.DETECTED.lastModified(asFile());
+					File file = asFile();
+					if (file != null)
+						return FS.DETECTED.lastModified(file);
+					return 0;
 				} catch (IOException e) {
 					return 0;
 				}
@@ -369,9 +377,13 @@ public class ContainerTreeIterator extends WorkingTreeIterator {
 
 		@Override
 		public InputStream openInputStream() throws IOException {
-			if (mode == FileMode.SYMLINK) {
-				return new ByteArrayInputStream(FS.DETECTED.readSymLink(
-						asFile()).getBytes(Constants.CHARACTER_ENCODING));
+			if (fileMode == FileMode.SYMLINK) {
+				File file = asFile();
+				if (file == null)
+					throw new IOException(MessageFormat.format(
+							CoreText.ContainerTreeIterator_DeletedFile, rsrc));
+				return new ByteArrayInputStream(FS.DETECTED.readSymLink(file)
+						.getBytes(Constants.CHARACTER_ENCODING));
 			} else {
 				if (rsrc.getType() == IResource.FILE)
 					try {
