@@ -19,7 +19,6 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.TreeSet;
-import java.util.Vector;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.locks.ReentrantLock;
 
@@ -35,14 +34,13 @@ import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.OperationCanceledException;
 import org.eclipse.core.runtime.Status;
-import org.eclipse.core.runtime.jobs.IJobChangeEvent;
 import org.eclipse.core.runtime.jobs.Job;
-import org.eclipse.core.runtime.jobs.JobChangeAdapter;
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.EclipseGitProgressTransformer;
 import org.eclipse.egit.core.IteratorService;
 import org.eclipse.egit.core.JobFamilies;
 import org.eclipse.egit.core.internal.CoreText;
+import org.eclipse.egit.core.internal.job.RuleUtil;
 import org.eclipse.egit.core.internal.trace.GitTraceLocation;
 import org.eclipse.egit.core.internal.util.ProjectUtil;
 import org.eclipse.jgit.dircache.DirCache;
@@ -79,7 +77,7 @@ public class IndexDiffCacheEntry {
 
 	private volatile boolean reloadJobIsInitializing;
 
-	private Vector<Job> updateJobs = new Vector<Job>();
+	private IndexDiffUpdateJob updateJob;
 
 	private DirCache lastIndex;
 
@@ -147,13 +145,14 @@ public class IndexDiffCacheEntry {
 	}
 
 	/**
-	 * This method starts a Job that refreshes all open projects related to the
-	 * repository and afterwards triggers the (asynchronous) recalculation of
-	 * the IndexDiff. This ensures that the IndexDiff calculation is not working
-	 * on out-dated resources.
+	 * This method creates (but does not start) a {@link Job} that refreshes all
+	 * open projects related to the repository and afterwards triggers the
+	 * (asynchronous) recalculation of the {@link IndexDiff}. This ensures that
+	 * the {@link IndexDiff} calculation is not working on out-dated resources.
 	 *
+	 * @return new job ready to be scheduled, never null
 	 */
-	public void refreshResourcesAndIndexDiff() {
+	public Job createRefreshResourcesAndIndexDiffJob() {
 		String repositoryName = Activator.getDefault().getRepositoryUtil()
 				.getRepositoryName(repository);
 		String jobName = MessageFormat
@@ -175,8 +174,8 @@ public class IndexDiffCacheEntry {
 			}
 
 		};
-		job.setRule(ResourcesPlugin.getWorkspace().getRoot());
-		job.schedule();
+		job.setRule(RuleUtil.getRule(repository));
+		return job;
 	}
 
 	/**
@@ -257,35 +256,45 @@ public class IndexDiffCacheEntry {
 		return indexDiffData;
 	}
 
-	private void scheduleReloadJob(final String trigger) {
+	/**
+	 * THIS METHOD IS PROTECTED FOR TESTS ONLY!
+	 *
+	 * @param trigger
+	 */
+	protected void scheduleReloadJob(final String trigger) {
 		if (reloadJob != null) {
-			if (reloadJobIsInitializing)
+			if (reloadJobIsInitializing) {
 				return;
+			}
 			reloadJob.cancel();
 		}
-		for (Job updateJob : updateJobs.toArray(new Job[updateJobs.size()]))
-			updateJob.cancel();
+		if (updateJob != null) {
+			updateJob.cleanupAndCancel();
+		}
 
-		if (!checkRepository())
+		if (!checkRepository()) {
 			return;
+		}
 		reloadJob = new Job(getReloadJobName()) {
 			@Override
 			protected IStatus run(IProgressMonitor monitor) {
 				try {
 					reloadJobIsInitializing = true;
 					waitForWorkspaceLock(monitor);
-					lock.lock();
 				} finally {
 					reloadJobIsInitializing = false;
 				}
+				lock.lock();
 				try {
-					if (monitor.isCanceled())
+					if (monitor.isCanceled()) {
 						return Status.CANCEL_STATUS;
+					}
 					parallelism.acquire();
 					long startTime = System.currentTimeMillis();
 					IndexDiffData result = calcIndexDiffDataFull(monitor, getName());
-					if (monitor.isCanceled() || (result == null))
+					if (monitor.isCanceled() || (result == null)) {
 						return Status.CANCEL_STATUS;
+					}
 					indexDiffData = result;
 					if (GitTraceLocation.INDEXDIFFCACHE.isActive()) {
 						long time = System.currentTimeMillis() - startTime;
@@ -321,8 +330,9 @@ public class IndexDiffCacheEntry {
 
 			@Override
 			public boolean belongsTo(Object family) {
-				if (family.equals(JobFamilies.INDEX_DIFF_CACHE_UPDATE))
+				if (JobFamilies.INDEX_DIFF_CACHE_UPDATE.equals(family)) {
 					return true;
+				}
 				return super.belongsTo(family);
 			}
 
@@ -354,32 +364,67 @@ public class IndexDiffCacheEntry {
 		}
 	}
 
-	private void scheduleUpdateJob(final Collection<String> filesToUpdate,
+	/**
+	 * THIS METHOD IS PROTECTED FOR TESTS ONLY!
+	 *
+	 * @param filesToUpdate
+	 * @param resourcesToUpdate
+	 */
+	protected void scheduleUpdateJob(final Collection<String> filesToUpdate,
 			final Collection<IResource> resourcesToUpdate) {
 		if (!checkRepository())
 			return;
 		if (reloadJob != null && reloadJobIsInitializing)
 			return;
-		Job job = new Job(getUpdateJobName()) {
+
+		if (shouldReload(filesToUpdate)) {
+			// Calculate new IndexDiff if too many resources changed
+			// This happens e.g. when a project is opened
+			scheduleReloadJob("Too many resources changed: " + filesToUpdate.size()); //$NON-NLS-1$
+			return;
+		}
+
+		if (updateJob != null) {
+			updateJob.addChanges(filesToUpdate, resourcesToUpdate);
+			return;
+		}
+		updateJob = new IndexDiffUpdateJob(getUpdateJobName(), 10) {
 			@Override
-			protected IStatus run(IProgressMonitor monitor) {
-				waitForWorkspaceLock(monitor);
-				if (monitor.isCanceled())
+			protected IStatus updateIndexDiff(Collection<String> files,
+					Collection<IResource> resources,
+					IProgressMonitor monitor) {
+				if (monitor.isCanceled()) {
 					return Status.CANCEL_STATUS;
+				}
+
+				// second check here is required because we collect changes
+				if (shouldReload(files)) {
+					// Calculate new IndexDiff if too many resources changed
+					// This happens e.g. when a project is opened
+					scheduleReloadJob("Too many resources changed: " + files.size()); //$NON-NLS-1$
+					return Status.CANCEL_STATUS;
+				}
+
+				waitForWorkspaceLock(monitor);
+
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
 				lock.lock();
 				try {
 					long startTime = System.currentTimeMillis();
 					IndexDiffData result = calcIndexDiffDataIncremental(monitor,
-							getName(), filesToUpdate, resourcesToUpdate);
-					if (monitor.isCanceled() || (result == null))
+							getName(), files, resources);
+					if (monitor.isCanceled() || (result == null)) {
 						return Status.CANCEL_STATUS;
+					}
 					indexDiffData = result;
 					if (GitTraceLocation.INDEXDIFFCACHE.isActive()) {
 						long time = System.currentTimeMillis() - startTime;
 						StringBuilder message = new StringBuilder(
 								NLS.bind(
 										"Updated IndexDiffData based on resource list (length = {0}) in {1} ms\n", //$NON-NLS-1$
-										Integer.valueOf(resourcesToUpdate
+										Integer.valueOf(resources
 												.size()), Long.valueOf(time)));
 						GitTraceLocation.getTrace().trace(
 								GitTraceLocation.INDEXDIFFCACHE.getLocation(),
@@ -389,10 +434,11 @@ public class IndexDiffCacheEntry {
 					notifyListeners();
 					return Status.OK_STATUS;
 				} catch (IOException e) {
-					if (GitTraceLocation.INDEXDIFFCACHE.isActive())
+					if (GitTraceLocation.INDEXDIFFCACHE.isActive()) {
 						GitTraceLocation.getTrace().trace(
 								GitTraceLocation.INDEXDIFFCACHE.getLocation(),
 								"Calculating IndexDiff failed", e); //$NON-NLS-1$
+					}
 					return Status.OK_STATUS;
 				} finally {
 					lock.unlock();
@@ -400,19 +446,25 @@ public class IndexDiffCacheEntry {
 			}
 			@Override
 			public boolean belongsTo(Object family) {
-				if (family.equals(JobFamilies.INDEX_DIFF_CACHE_UPDATE))
+				if (JobFamilies.INDEX_DIFF_CACHE_UPDATE.equals(family)) {
 					return true;
+				}
 				return super.belongsTo(family);
 			}
 
 		};
-		updateJobs.add(job);
-		job.addJobChangeListener(new JobChangeAdapter() {
-			public void done(IJobChangeEvent event) {
-				updateJobs.remove(event.getJob());
-			}
-		});
-		job.schedule();
+
+		updateJob.addChanges(filesToUpdate, resourcesToUpdate);
+	}
+
+	/**
+	 * Check if the index update or reload is recommended for given files
+	 *
+	 * @param filesToUpdate
+	 * @return true if the reload operation is preferred
+	 */
+	protected boolean shouldReload(final Collection<String> filesToUpdate) {
+		return filesToUpdate.size() > RESOURCE_LIST_UPDATE_LIMIT;
 	}
 
 	private IndexDiffData calcIndexDiffDataIncremental(IProgressMonitor monitor,
@@ -511,17 +563,14 @@ public class IndexDiffCacheEntry {
 					return;
 				}
 				Collection<String> filesToUpdate = visitor.getFilesToUpdate();
-				if (visitor.getGitIgnoreChanged())
+				if (visitor.getGitIgnoreChanged()) {
 					scheduleReloadJob("A .gitignore changed"); //$NON-NLS-1$
-				else if (indexDiffData == null)
+				} else if (indexDiffData == null) {
 					scheduleReloadJob("Resource changed, no diff available"); //$NON-NLS-1$
-				else if (!filesToUpdate.isEmpty())
-					if (filesToUpdate.size() < RESOURCE_LIST_UPDATE_LIMIT)
-						scheduleUpdateJob(filesToUpdate, visitor.getResourcesToUpdate());
-					else
-						// Calculate new IndexDiff if too many resources changed
-						// This happens e.g. when a project is opened
-						scheduleReloadJob("Too many resources changed"); //$NON-NLS-1$
+				} else if (!filesToUpdate.isEmpty()) {
+					scheduleUpdateJob(filesToUpdate,
+							visitor.getResourcesToUpdate());
+				}
 			}
 
 		};
@@ -529,6 +578,15 @@ public class IndexDiffCacheEntry {
 				resourceChangeListener, IResourceChangeEvent.POST_CHANGE);
 	}
 
+	/**
+	 * FOR TESTS ONLY
+	 *
+	 * @return job used to schedule incremental updates
+	 */
+	protected IndexDiffUpdateJob getUpdateJob() {
+		return updateJob;
+	}
+
 	/**
 	 * Dispose cache entry by removing listeners.
 	 */
