@@ -11,7 +11,11 @@
  *******************************************************************************/
 package org.eclipse.egit.core.internal.util;
 
+import java.io.ByteArrayInputStream;
+import java.io.File;
+import java.io.IOException;
 import java.net.URI;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -37,8 +41,12 @@ import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.GitProvider;
 import org.eclipse.egit.core.RepositoryCache;
+import org.eclipse.egit.core.internal.CoreText;
+import org.eclipse.egit.core.internal.indexdiff.IndexDiffCacheEntry;
+import org.eclipse.egit.core.internal.indexdiff.IndexDiffData;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.jgit.lib.Repository;
+import org.eclipse.jgit.util.FS;
 import org.eclipse.team.core.RepositoryProvider;
 
 /**
@@ -136,6 +144,28 @@ public class ResourceUtil {
 		return root.getContainerForLocation(path);
 	}
 
+	/**
+	 * Checks if the path relative to the given repository refers to a symbolic
+	 * link
+	 *
+	 * @param repository
+	 *            the repository of the file
+	 * @param repoRelativePath
+	 *            the repository-relative path of the file to search for
+	 * @return {@code true} if the path in the given repository refers to a
+	 *         symbolic link
+	 */
+	public static boolean isSymbolicLink(Repository repository,
+			String repoRelativePath) {
+		try {
+			File f = new Path(repository.getWorkTree().getAbsolutePath())
+					.append((repoRelativePath)).toFile();
+			return FS.DETECTED.isSymLink(f);
+		} catch (@SuppressWarnings("unused") IOException e) {
+			return false;
+		}
+	}
+
 	/**
 	 * The method splits the given resources by their repository. For each
 	 * occurring repository a list is built containing the repository relative
@@ -292,4 +322,48 @@ public class ResourceUtil {
 		}
 		return mappings.toArray(new ResourceMapping[mappings.size()]);
 	}
+
+	/**
+	 * Save local history.
+	 *
+	 * @param repository
+	 */
+	public static void saveLocalHistory(Repository repository) {
+		IndexDiffCacheEntry indexDiffCacheEntry = org.eclipse.egit.core.Activator
+				.getDefault().getIndexDiffCache()
+				.getIndexDiffCacheEntry(repository);
+		IndexDiffData indexDiffData = indexDiffCacheEntry.getIndexDiff();
+		if (indexDiffData != null) {
+			Collection<IResource> changedResources = indexDiffData
+					.getChangedResources();
+			if (changedResources != null) {
+				for (IResource changedResource : changedResources) {
+					if (changedResource instanceof IFile
+							&& changedResource.exists()) {
+						try {
+							ResourceUtil.saveLocalHistory(changedResource);
+						} catch (CoreException e) {
+							// Ignore error. Failure to save local history must
+							// not interfere with the operation.
+							Activator
+									.logError(
+											MessageFormat
+													.format(CoreText.ResourceUtil_SaveLocalHistoryFailed,
+															changedResource), e);
+						}
+					}
+				}
+			}
+		}
+	}
+
+	private static void saveLocalHistory(IResource resource)
+			throws CoreException {
+		if (!resource.isSynchronized(IResource.DEPTH_ZERO))
+			resource.refreshLocal(IResource.DEPTH_ZERO, null);
+		// Dummy update to force save for local history.
+		((IFile) resource).appendContents(
+				new ByteArrayInputStream(new byte[0]), IResource.KEEP_HISTORY,
+				null);
+	}
 }
