@@ -530,11 +530,8 @@ public class GitProjectData {
 				CoreText.GitProjectData_mappedResourceGone, m.toString()),
 				new FileNotFoundException(m.getContainerPath().toString()));
 		m.clear();
-		try {
-			RepositoryProvider.unmap(getProject());
-		} catch (TeamException e) {
-			Activator.logError(CoreText.GitProjectData_UnmappingGoneResourceFailed, e);
-		}
+		UnmapJob unmapJob = new UnmapJob(getProject());
+		unmapJob.schedule();
 	}
 
 	private void protect(IResource resource) {
@@ -553,4 +550,26 @@ public class GitProjectData {
 			c = c.getParent();
 		}
 	}
+
+	private static class UnmapJob extends Job {
+
+		private final IProject project;
+
+		private UnmapJob(IProject project) {
+			super(MessageFormat.format(CoreText.GitProjectData_UnmapJobName,
+					project.getName()));
+			this.project = project;
+		}
+
+		@Override
+		protected IStatus run(IProgressMonitor monitor) {
+			try {
+				RepositoryProvider.unmap(project);
+				return Status.OK_STATUS;
+			} catch (TeamException e) {
+				return new Status(IStatus.ERROR, Activator.getPluginId(),
+						CoreText.GitProjectData_UnmappingGoneResourceFailed, e);
+			}
+		}
+	}
 }
