@@ -18,8 +18,8 @@ import org.eclipse.egit.ui.JobFamilies;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.test.ContextMenuHelper;
 import org.eclipse.egit.ui.test.JobJoiner;
-import org.eclipse.egit.ui.test.TestUtil;
 import org.eclipse.jface.dialogs.IDialogConstants;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.swtbot.eclipse.finder.SWTWorkbenchBot;
 import org.eclipse.swtbot.swt.finder.SWTBot;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotCombo;
@@ -32,9 +32,8 @@ public class PushBranchWizardTester {
 
 	public static PushBranchWizardTester startWizard(SWTBotTree projectTree,
 			String branchName) {
-		TestUtil util = new TestUtil();
-		String pushBranchMenu = util
-				.getPluginLocalizedValue("PushBranchAction.label");
+		String pushBranchMenu = NLS
+				.bind(UIText.PushMenu_PushBranch, branchName);
 		ContextMenuHelper.clickContextMenu(projectTree, "Team", pushBranchMenu);
 		return forBranchName(branchName);
 	}
@@ -89,13 +88,15 @@ public class PushBranchWizardTester {
 	}
 
 	public void enterBranchName(String branchName) {
-		wizard.textWithLabel(UIText.PushBranchPage_BranchNameLabel).setText(
+		wizard.textWithLabel(UIText.PushBranchPage_RemoteBranchNameLabel)
+				.setText(
 				branchName);
 	}
 
 	public void assertBranchName(String branchName) {
 		assertEquals(branchName,
-				wizard.textWithLabel(UIText.PushBranchPage_BranchNameLabel)
+				wizard.textWithLabel(
+						UIText.PushBranchPage_RemoteBranchNameLabel)
 						.getText());
 	}
 
@@ -116,10 +117,20 @@ public class PushBranchWizardTester {
 		wizard.radio(UIText.UpstreamConfigComponent_RebaseRadio).click();
 	}
 
-	public void assertRebaseSelected() {
+	public void assertConfigureUpstreamSelected() {
 		assertTrue(wizard.checkBox(
 				UIText.UpstreamConfigComponent_ConfigureUpstreamCheck)
 				.isChecked());
+	}
+
+	public void assertMergeSelected() {
+		assertConfigureUpstreamSelected();
+		assertTrue(wizard.radio(UIText.UpstreamConfigComponent_MergeRadio)
+				.isSelected());
+	}
+
+	public void assertRebaseSelected() {
+		assertConfigureUpstreamSelected();
 		assertTrue(wizard.radio(UIText.UpstreamConfigComponent_RebaseRadio)
 				.isSelected());
 	}
