@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 SAP AG.
+ * Copyright (c) 2010, 2013 SAP AG and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -32,7 +32,6 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.jobs.Job;
-import org.eclipse.egit.core.op.BranchOperation;
 import org.eclipse.egit.core.op.CommitOperation;
 import org.eclipse.egit.core.op.TagOperation;
 import org.eclipse.egit.ui.Activator;
@@ -56,11 +55,11 @@ import org.eclipse.jgit.util.FileUtils;
 import org.eclipse.jgit.util.RawParseUtils;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Display;
-import org.eclipse.swtbot.eclipse.finder.widgets.SWTBotPerspective;
 import org.eclipse.swtbot.swt.finder.SWTBot;
 import org.eclipse.swtbot.swt.finder.junit.SWTBotJunit4ClassRunner;
 import org.eclipse.swtbot.swt.finder.utils.TableCollection;
 import org.eclipse.swtbot.swt.finder.waits.ICondition;
+import org.eclipse.swtbot.swt.finder.widgets.SWTBotLabel;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotShell;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotToolbarDropDownButton;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;
@@ -69,9 +68,7 @@ import org.eclipse.swtbot.swt.finder.widgets.TimeoutException;
 import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.ide.IDE.SharedImages;
-import org.junit.AfterClass;
 import org.junit.Before;
-import org.junit.BeforeClass;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 
@@ -80,20 +77,16 @@ import org.junit.runner.RunWith;
  */
 @RunWith(SWTBotJunit4ClassRunner.class)
 public class BranchAndResetActionTest extends LocalRepositoryTestCase {
-	private static File repositoryFile;
-
-	private static SWTBotPerspective perspective;
 
 	private static String LOCAL_BRANCHES;
-
 	private static String TAGS;
 
-	@BeforeClass
-	public static void setup() throws Exception {
+	private File repositoryFile;
+
+	@Before
+	public void setup() throws Exception {
 		repositoryFile = createProjectAndCommitToRepository();
 		Repository repo = lookupRepository(repositoryFile);
-		perspective = bot.activePerspective();
-		bot.perspectiveById("org.eclipse.pde.ui.PDEPerspective").activate();
 
 		TagBuilder tag = new TagBuilder();
 		tag.setTag("SomeTag");
@@ -111,21 +104,6 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 				null, repo), repo));
 		TAGS = provider.getText(new TagsNode(new RepositoryNode(null, repo),
 				repo));
-		waitInUI();
-	}
-
-	@AfterClass
-	public static void shutdown() {
-		perspective.activate();
-	}
-
-	@Before
-	public void prepare() throws Exception {
-		Repository repo = lookupRepository(repositoryFile);
-		if (!repo.getBranch().equals("master")) {
-			BranchOperation bop = new BranchOperation(repo, "refs/heads/master");
-			bop.execute(null);
-		}
 	}
 
 	@Test
@@ -287,8 +265,9 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 		String master = getTestFileContent();
 		assertFalse(stable.equals(master));
 		SWTBotShell resetDialog = openResetDialog();
-		resetDialog.bot().tree().getTreeItem(LOCAL_BRANCHES).getNode("stable")
-				.select();
+		SWTBotTreeItem localBranches = resetDialog.bot().tree()
+				.getTreeItem(LOCAL_BRANCHES).expand();
+		TestUtil.getChildNode(localBranches, "stable").select();
 		resetDialog.bot().radio(
 				UIText.ResetTargetSelectionDialog_ResetTypeHardButton).click();
 
@@ -308,7 +287,6 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 		assertNull(lookupRepository(repositoryFile).resolve("newBranch"));
 
 		SWTBotShell newBranchDialog = openCreateBranchDialog();
-		newBranchDialog.bot().comboBoxWithId("BaseBranch").setSelection(0);
 		newBranchDialog.bot().textWithId("BranchName").setText("newBranch");
 		newBranchDialog.bot().checkBox(UIText.CreateBranchPage_CheckoutButton).deselect();
 		newBranchDialog.bot().button(IDialogConstants.FINISH_LABEL).click();
@@ -317,29 +295,62 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 		assertNotNull(lookupRepository(repositoryFile).resolve("newBranch"));
 
 		SWTBotShell deleteBranchDialog = openDeleteBranchDialog();
-		deleteBranchDialog.bot().tree().getTreeItem(LOCAL_BRANCHES).expand().getNode("newBranch").select();
+		SWTBotTreeItem localBranches = deleteBranchDialog.bot().tree()
+				.getTreeItem(LOCAL_BRANCHES).expand();
+		TestUtil.getChildNode(localBranches, "newBranch").select();
 		deleteBranchDialog.bot().button(IDialogConstants.OK_LABEL).click();
 
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
 		assertNull(lookupRepository(repositoryFile).resolve("newBranch"));
 	}
 
+	@Test
+	public void testCreateBranchSelectSource() throws Exception {
+		Repository repo = lookupRepository(repositoryFile);
+		assertNull(repo.resolve("branch-from-tag"));
+
+		SWTBotShell createBranchDialog = openCreateBranchDialog();
+		createBranchDialog.bot()
+				.button(UIText.CreateBranchPage_SourceSelectButton).click();
+
+		SWTBotShell sourceSelectionDialog = bot
+				.shell(UIText.CreateBranchPage_SourceSelectionDialogTitle);
+		SWTBotTree tree = sourceSelectionDialog.bot().tree();
+		SWTBotTreeItem tags = tree
+				.expandNode(UIText.RepositoriesViewLabelProvider_TagsNodeText);
+		TestUtil.getChildNode(tags, "SomeTag").select();
+		sourceSelectionDialog.bot().button(IDialogConstants.OK_LABEL).click();
+
+		SWTBotLabel sourceLabel = createBranchDialog.bot().label(3);
+		assertEquals("SomeTag", sourceLabel.getText());
+
+		createBranchDialog.bot().textWithId("BranchName")
+				.setText("branch-from-tag");
+		createBranchDialog.bot()
+				.checkBox(UIText.CreateBranchPage_CheckoutButton).deselect();
+		createBranchDialog.bot().button(IDialogConstants.FINISH_LABEL).click();
+
+		ObjectId resolvedBranch = repo.resolve("branch-from-tag");
+		ObjectId resolvedTagCommit = repo.resolve("SomeTag^{commit}");
+		assertNotNull(resolvedBranch);
+		assertEquals(resolvedTagCommit, resolvedBranch);
+	}
+
 	private SWTBotShell openCheckoutBranchDialog() {
-		SWTBotTree projectExplorerTree = bot.viewById(
-				"org.eclipse.jdt.ui.PackageExplorer").bot().tree();
+		SWTBotTree projectExplorerTree = TestUtil.getExplorerTree();
 		getProjectItem(projectExplorerTree, PROJ1).select();
 		String[] menuPath = new String[] {
 				util.getPluginLocalizedValue("TeamMenu.label"),
 				util.getPluginLocalizedValue("SwitchToMenu.label"),
 				UIText.SwitchToMenu_OtherMenuLabel };
 		ContextMenuHelper.clickContextMenu(projectExplorerTree, menuPath);
-		SWTBotShell dialog = bot.shell(UIText.CheckoutDialog_WindowTitle);
+		SWTBotShell dialog = bot
+				.shell(UIText.BranchSelectionAndEditDialog_WindowTitle);
 		return dialog;
 	}
 
 	private SWTBotShell openCreateBranchDialog() {
-			SWTBotTree projectExplorerTree = bot.viewById(
-					"org.eclipse.jdt.ui.PackageExplorer").bot().tree();
+		SWTBotTree projectExplorerTree = TestUtil.getExplorerTree();
 			getProjectItem(projectExplorerTree, PROJ1).select();
 			String[] menuPath = new String[] {
 					util.getPluginLocalizedValue("TeamMenu.label"),
@@ -348,12 +359,11 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 			ContextMenuHelper.clickContextMenu(projectExplorerTree, menuPath);
 		SWTBotShell dialog = bot
 				.shell(UIText.CreateBranchWizard_NewBranchTitle);
-			return dialog;
-		}
+		return dialog;
+	}
 
 	private SWTBotShell openRenameBranchDialog() {
-		SWTBotTree projectExplorerTree = bot.viewById(
-				"org.eclipse.jdt.ui.PackageExplorer").bot().tree();
+		SWTBotTree projectExplorerTree = TestUtil.getExplorerTree();
 		getProjectItem(projectExplorerTree, PROJ1).select();
 		String[] menuPath = new String[] {
 				util.getPluginLocalizedValue("TeamMenu.label"),
@@ -365,8 +375,7 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 	}
 
 	private SWTBotShell openDeleteBranchDialog() {
-		SWTBotTree projectExplorerTree = bot.viewById(
-				"org.eclipse.jdt.ui.PackageExplorer").bot().tree();
+		SWTBotTree projectExplorerTree = TestUtil.getExplorerTree();
 		getProjectItem(projectExplorerTree, PROJ1).select();
 		String[] menuPath = new String[] {
 				util.getPluginLocalizedValue("TeamMenu.label"),
@@ -378,8 +387,7 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 	}
 
 	private SWTBotShell openResetDialog() {
-		SWTBotTree projectExplorerTree = bot.viewById(
-				"org.eclipse.jdt.ui.PackageExplorer").bot().tree();
+		SWTBotTree projectExplorerTree = TestUtil.getExplorerTree();
 		getProjectItem(projectExplorerTree, PROJ1).select();
 		String menuString = util.getPluginLocalizedValue("ResetAction_label");
 		ContextMenuHelper.clickContextMenu(projectExplorerTree, util
@@ -393,8 +401,9 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 	public void testRenameBranch() throws Exception {
 		SWTBotShell dialog = openRenameBranchDialog();
 
-		dialog.bot().tree().getTreeItem(LOCAL_BRANCHES).expand()
-				.getNode("stable").select();
+		SWTBotTreeItem localBranches = dialog.bot().tree()
+				.getTreeItem(LOCAL_BRANCHES).expand();
+		TestUtil.getChildNode(localBranches, "stable").select();
 		dialog.bot().button(UIText.RenameBranchDialog_RenameButtonLabel)
 				.click();
 		// rename stable to renamed
@@ -408,14 +417,10 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 		newNameDialog.bot().button(IDialogConstants.OK_LABEL).click();
 
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
-		dialog = openRenameBranchDialog();
-		dialog.bot().tree().getTreeItem(LOCAL_BRANCHES).expand()
-				.getNode("renamed");
-		dialog.close();
 
 		dialog = openRenameBranchDialog();
-		dialog.bot().tree().getTreeItem(LOCAL_BRANCHES).expand()
-				.getNode("renamed").select();
+		SWTBotTreeItem localBranches2 = dialog.bot().tree().getTreeItem(LOCAL_BRANCHES).expand();
+		TestUtil.getChildNode(localBranches2, "renamed").select();
 		dialog.bot().button(UIText.RenameBranchDialog_RenameButtonLabel)
 				.click();
 		// rename renamed to stable
@@ -427,8 +432,9 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
 		dialog = openRenameBranchDialog();
-		dialog.bot().tree().getTreeItem(LOCAL_BRANCHES).expand()
-				.getNode("stable");
+		SWTBotTreeItem localBranches3 = dialog.bot().tree()
+				.getTreeItem(LOCAL_BRANCHES).expand();
+		TestUtil.getChildNode(localBranches3, "stable").select();
 		dialog.close();
 	}
 
@@ -437,17 +443,18 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 		checkoutAndVerify(new String[] { TAGS, "SomeTag" });
 	}
 
-	private void checkoutAndVerify(String[] newBranch)
+	private void checkoutAndVerify(String[] nodeTexts)
 			throws IOException, Exception {
 		SWTBotShell dialog = openCheckoutBranchDialog();
 		TableCollection tc = dialog.bot().tree().selection();
 		assertEquals("Wrong selection count", 0, tc.rowCount());
 
-		dialog.bot().tree().getTreeItem(newBranch[0]).expand().getNode(
-				newBranch[1]).select();
+		SWTBotTreeItem parentNode = dialog.bot().tree()
+				.getTreeItem(nodeTexts[0]).expand();
+		TestUtil.getChildNode(parentNode, nodeTexts[1]).select();
 		tc = dialog.bot().tree().selection();
 		assertEquals("Wrong selection count", 1, tc.rowCount());
-		assertEquals("Wrong item selected", newBranch[1], tc.get(0, 0));
+		assertTrue("Wrong item selected", tc.get(0, 0).startsWith(nodeTexts[1]));
 
 		Repository repo = lookupRepository(repositoryFile);
 
@@ -456,19 +463,21 @@ public class BranchAndResetActionTest extends LocalRepositoryTestCase {
 		if (ObjectId.isId(repo.getBranch())) {
 			String mapped = Activator.getDefault().getRepositoryUtil()
 					.mapCommitToRef(repo, repo.getBranch(), false);
-			assertEquals("Wrong branch", newBranch[1], mapped.substring(mapped
+			assertEquals("Wrong branch", nodeTexts[1],
+					mapped.substring(mapped
 					.lastIndexOf('/') + 1));
 		} else
-			assertEquals("Wrong branch", newBranch[1], repo.getBranch());
+			assertEquals("Wrong branch", nodeTexts[1], repo.getBranch());
 	}
 
-	private void checkout(String[] newBranch) throws Exception {
+	private void checkout(String[] nodeTexts) throws Exception {
 		SWTBotShell dialog = openCheckoutBranchDialog();
-		dialog.bot().tree().getTreeItem(newBranch[0]).expand().getNode(
-				newBranch[1]).select();
+		SWTBotTreeItem parentNode = dialog.bot().tree()
+				.getTreeItem(nodeTexts[0]).expand();
+		TestUtil.getChildNode(parentNode, nodeTexts[1]).select();
 		TableCollection tc = dialog.bot().tree().selection();
 		assertEquals("Wrong selection count", 1, tc.rowCount());
-		assertEquals("Wrong item selected", newBranch[1], tc.get(0, 0));
+		assertTrue("Wrong item selected", tc.get(0, 0).startsWith(nodeTexts[1]));
 
 		dialog.bot().button(UIText.CheckoutDialog_OkCheckout).click();
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
