@@ -317,10 +317,7 @@ public class GitRepositoriesViewRepoHandlingTest extends
 		SWTBotShell shell = bot.shell(UIText.NewRepositoryWizard_WizardTitle);
 		IPath newPath = new Path(getTestDirectory().getPath());
 		shell.bot().textWithLabel(UIText.CreateRepositoryPage_DirectoryLabel)
-				.setText(newPath.toOSString());
-		shell.bot()
-				.textWithLabel(UIText.CreateRepositoryPage_RepositoryNameLabel)
-				.setText("NewRepository");
+				.setText(newPath.append("NewRepository").toOSString());
 		shell.bot().button(IDialogConstants.FINISH_LABEL).click();
 		refreshAndWait();
 		File repoFile = new File(newPath.append("NewRepository").toFile(),
@@ -334,16 +331,14 @@ public class GitRepositoriesViewRepoHandlingTest extends
 						myUtil.getPluginLocalizedValue("RepoViewCreateRepository.tooltip"))
 				.click();
 		shell = bot.shell(UIText.NewRepositoryWizard_WizardTitle);
-		newPath = new Path(getTestDirectory().getPath()).append("bare");
-		shell.bot()
-				.textWithLabel(UIText.CreateRepositoryPage_RepositoryNameLabel)
-				.setText("NewBareRepository");
+		newPath = new Path(getTestDirectory().getPath()).append("bare").append(
+				"NewBareRepository");
 		shell.bot().textWithLabel(UIText.CreateRepositoryPage_DirectoryLabel)
 				.setText(newPath.toOSString());
 		shell.bot().checkBox(UIText.CreateRepositoryPage_BareCheckbox).select();
 		shell.bot().button(IDialogConstants.FINISH_LABEL).click();
 		refreshAndWait();
-		repoFile = newPath.append("NewBareRepository").toFile();
+		repoFile = newPath.toFile();
 		myRepoViewUtil.getRootItem(getOrOpenView().bot().tree(), repoFile);
 		assertTrue(myRepoViewUtil.lookupRepository(repoFile).isBare());
 	}
