@@ -22,7 +22,6 @@ import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
-import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.eclipse.core.net.proxy.IProxyService;
 import org.eclipse.core.resources.IProject;
@@ -35,6 +34,8 @@ import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.core.runtime.SubProgressMonitor;
 import org.eclipse.core.runtime.jobs.ISchedulingRule;
+import org.eclipse.core.runtime.jobs.Job;
+import org.eclipse.egit.core.RepositoryCache;
 import org.eclipse.egit.core.RepositoryUtil;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.internal.ConfigurationChecker;
@@ -196,6 +197,7 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 	private ListenerHandle refreshHandle;
 	private DebugOptions debugOptions;
 
+	private volatile boolean uiIsActive;
 	private IWindowListener focusListener;
 
 	/**
@@ -233,41 +235,69 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 		CredentialsProvider.setDefault(new EGitCredentialsProvider());
 	}
 
+	/**
+	 * @return true if at least one Eclipse window is active
+	 */
 	static boolean isActive() {
-		final AtomicBoolean ret = new AtomicBoolean();
-		final Display display = PlatformUI.getWorkbench().getDisplay();
-		if (display.isDisposed())
-			return false;
-		display.syncExec(new Runnable() {
-			public void run() {
-				ret.set(display.getActiveShell() != null);
-			}
-		});
-		return ret.get();
+		return getDefault().uiIsActive;
 	}
 
+
 	private void setupFocusHandling() {
 		focusListener = new IWindowListener() {
 
+			private void updateUiState() {
+				Display.getCurrent().asyncExec(new Runnable() {
+					public void run() {
+						boolean wasActive = uiIsActive;
+						uiIsActive = Display.getCurrent().getActiveShell() != null;
+						if (uiIsActive != wasActive
+								&& GitTraceLocation.REPOSITORYCHANGESCANNER
+										.isActive())
+							traceUiIsActive();
+					}
+
+					private void traceUiIsActive() {
+						StringBuilder message = new StringBuilder(
+								"workbench is "); //$NON-NLS-1$
+						message.append(uiIsActive ? "active" : "inactive"); //$NON-NLS-1$//$NON-NLS-2$
+						GitTraceLocation.getTrace().trace(
+								GitTraceLocation.REPOSITORYCHANGESCANNER
+										.getLocation(), message.toString());
+					}
+				});
+			}
+
 			public void windowOpened(IWorkbenchWindow window) {
-				// nothing
+				updateUiState();
 			}
 
 			public void windowDeactivated(IWorkbenchWindow window) {
-				// nothing
+				updateUiState();
 			}
 
 			public void windowClosed(IWorkbenchWindow window) {
-				// nothing
+				updateUiState();
 			}
 
 			public void windowActivated(IWorkbenchWindow window) {
+				updateUiState();
 				if (rcs.doReschedule)
 					rcs.schedule();
 				refreshJob.triggerRefresh();
 			}
 		};
-		PlatformUI.getWorkbench().addWindowListener(focusListener);
+		Job job = new Job(UIText.Activator_setupFocusListener) {
+			@Override
+			protected IStatus run(IProgressMonitor monitor) {
+				if (PlatformUI.isWorkbenchRunning())
+					PlatformUI.getWorkbench().addWindowListener(focusListener);
+				else
+					schedule(1000L);
+				return Status.OK_STATUS;
+			}
+		};
+		job.schedule();
 	}
 
 	public void optionsChanged(DebugOptions options) {
@@ -422,7 +452,7 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 	 * A Job that looks at the repository meta data and triggers a refresh of
 	 * the resources in the affected projects.
 	 */
-	static class RepositoryChangeScanner extends WorkspaceJob {
+	static class RepositoryChangeScanner extends Job {
 		RepositoryChangeScanner() {
 			super(UIText.Activator_repoScanJobName);
 		}
@@ -437,9 +467,15 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 		}
 
 		@Override
-		public IStatus runInWorkspace(IProgressMonitor monitor) {
-			Repository[] repos = org.eclipse.egit.core.Activator.getDefault()
-					.getRepositoryCache().getAllRepositories();
+		protected IStatus run(IProgressMonitor monitor) {
+			// The core plugin might have been stopped before we could cancel
+			// this job.
+			RepositoryCache repositoryCache = org.eclipse.egit.core.Activator
+					.getDefault().getRepositoryCache();
+			if (repositoryCache == null)
+				return Status.OK_STATUS;
+
+			Repository[] repos = repositoryCache.getAllRepositories();
 			if (repos.length == 0)
 				return Status.OK_STATUS;
 
@@ -498,6 +534,7 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 		rcs.schedule(RepositoryChangeScanner.REPO_SCAN_INTERVAL);
 	}
 
+	@SuppressWarnings("unchecked")
 	private void setupSSH(final BundleContext context) {
 		final ServiceReference ssh;
 
@@ -527,7 +564,8 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 		}
 
 		if (focusListener != null) {
-			PlatformUI.getWorkbench().removeWindowListener(focusListener);
+			if (PlatformUI.isWorkbenchRunning())
+				PlatformUI.getWorkbench().removeWindowListener(focusListener);
 			focusListener = null;
 		}
 
