@@ -34,8 +34,12 @@ import org.eclipse.core.resources.mapping.ResourceTraversal;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Platform;
+import org.eclipse.core.runtime.Status;
+import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.core.runtime.preferences.DefaultScope;
 import org.eclipse.core.runtime.preferences.IEclipsePreferences;
 import org.eclipse.core.runtime.preferences.IEclipsePreferences.IPreferenceChangeListener;
@@ -50,8 +54,11 @@ import org.eclipse.egit.core.internal.util.ResourceUtil;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
-import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput.EmptyTypedElement;
 import org.eclipse.egit.ui.internal.merge.GitCompareEditorInput;
+import org.eclipse.egit.ui.internal.revision.EditableRevision;
+import org.eclipse.egit.ui.internal.revision.FileRevisionTypedElement;
+import org.eclipse.egit.ui.internal.revision.GitCompareFileRevisionEditorInput;
+import org.eclipse.egit.ui.internal.revision.GitCompareFileRevisionEditorInput.EmptyTypedElement;
 import org.eclipse.egit.ui.internal.synchronize.GitModelSynchronize;
 import org.eclipse.egit.ui.internal.synchronize.compare.LocalNonWorkspaceTypedElement;
 import org.eclipse.jface.action.Action;
@@ -76,6 +83,7 @@ import org.eclipse.jgit.treewalk.filter.TreeFilter;
 import org.eclipse.jgit.util.IO;
 import org.eclipse.jgit.util.io.EolCanonicalizingInputStream;
 import org.eclipse.osgi.util.NLS;
+import org.eclipse.swt.widgets.Display;
 import org.eclipse.team.core.history.IFileRevision;
 import org.eclipse.team.ui.synchronize.SaveableCompareEditorInput;
 import org.eclipse.ui.IEditorInput;
@@ -83,6 +91,7 @@ import org.eclipse.ui.IEditorPart;
 import org.eclipse.ui.IEditorReference;
 import org.eclipse.ui.IReusableEditor;
 import org.eclipse.ui.IWorkbenchPage;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
 
 /**
@@ -230,8 +239,8 @@ public class CompareUtils {
 	 * @return a truncated revision identifier if it is long
 	 */
 	public static String truncatedRevision(String ci) {
-		if (ci.length() > 10)
-			return ci.substring(0, 7) + "..."; //$NON-NLS-1$
+		if (ObjectId.isId(ci))
+			return ci.substring(0, 7);
 		else
 			return ci;
 	}
@@ -265,6 +274,35 @@ public class CompareUtils {
 		return null;
 	}
 
+	/**
+	 * Compares two files between the given commits, taking possible renames
+	 * into account.
+	 *
+	 * @param commit1
+	 *            the "left" commit for the comparison editor
+	 * @param commit2
+	 *            the "right" commit for the comparison editor
+	 * @param commit1Path
+	 *            path to the file within commit1's tree
+	 * @param commit2Path
+	 *            path to the file within commit2's tree
+	 * @param repository
+	 *            the repository this commit was loaded out of
+	 * @param workBenchPage
+	 *            the page to open the compare editor in
+	 */
+	public static void openInCompare(RevCommit commit1, RevCommit commit2,
+			String commit1Path, String commit2Path, Repository repository,
+			IWorkbenchPage workBenchPage) {
+		final ITypedElement base = CompareUtils.getFileRevisionTypedElement(
+				commit1Path, commit1, repository);
+		final ITypedElement next = CompareUtils.getFileRevisionTypedElement(
+				commit2Path, commit2, repository);
+		CompareEditorInput in = new GitCompareFileRevisionEditorInput(base,
+				next, null);
+		CompareUtils.openInCompare(workBenchPage, in);
+	}
+
 	/**
 	 * @param workBenchPage
 	 * @param input
@@ -410,24 +448,70 @@ public class CompareUtils {
 	 * @param page
 	 *            If not {@null} try to re-use a compare editor on this
 	 *            page if any is available. Otherwise open a new one.
-	 * @throws IOException
-	 *             If HEAD or {@code refName} can't be resolved in the given
-	 *             repository.
 	 */
-	private static void compareWorkspaceWithRef(Repository repository,
-			IFile file, String refName, IWorkbenchPage page) throws IOException {
-		final RepositoryMapping mapping = RepositoryMapping.getMapping(file);
-		final String gitPath = mapping.getRepoRelativePath(file);
-		final ITypedElement base = SaveableCompareEditorInput
-				.createFileElement(file);
+	private static void compareWorkspaceWithRef(final Repository repository,
+			final IFile file, final String refName, final IWorkbenchPage page) {
 
-		CompareEditorInput in = prepareCompareInput(repository, gitPath, base,
-				refName);
+		Job job = new Job(UIText.CompareUtils_jobName) {
 
-		if (page != null)
+			@Override
+			public IStatus run(IProgressMonitor monitor) {
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
+				final RepositoryMapping mapping = RepositoryMapping
+						.getMapping(file);
+				final String gitPath = mapping.getRepoRelativePath(file);
+				final ITypedElement base = SaveableCompareEditorInput
+						.createFileElement(file);
+
+				CompareEditorInput in;
+				try {
+					in = prepareCompareInput(repository, gitPath, base, refName);
+				} catch (IOException e) {
+					return Activator.createErrorStatus(
+							UIText.CompareWithRefAction_errorOnSynchronize, e);
+				}
+
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
+				openCompareEditorRunnable(page, in);
+				return Status.OK_STATUS;
+			}
+		};
+		job.setUser(true);
+		job.schedule();
+	}
+
+	/**
+	 * Opens compare editor in UI thread. Safe to start from background threads
+	 * too - in this case the operation will be started asynchronously in UI
+	 * thread.
+	 *
+	 * @param page
+	 *            can be null
+	 * @param in
+	 *            non null
+	 */
+	private static void openCompareEditorRunnable(
+			final IWorkbenchPage page,
+			final CompareEditorInput in) {
+		// safety check: make sure we open compare editor from UI thread
+		if (Display.getCurrent() == null) {
+			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
+				public void run() {
+					openCompareEditorRunnable(page, in);
+				}
+			});
+			return;
+		}
+
+		if (page != null) {
 			openInCompare(page, in);
-		else
+		} else {
 			CompareUI.openCompareEditor(in);
+		}
 	}
 
 	/**
@@ -446,23 +530,39 @@ public class CompareUtils {
 	 * @param page
 	 *            If not {@null} try to re-use a compare editor on this
 	 *            page if any is available. Otherwise open a new one.
-	 * @throws IOException
-	 *             If HEAD or {@code refName} can't be resolved in the given
-	 *             repository.
 	 */
-	private static void compareLocalWithRef(Repository repository,
-			IPath location, String refName, IWorkbenchPage page)
-			throws IOException {
-		final String gitPath = getRepoRelativePath(location, repository);
-		final ITypedElement base = new LocalNonWorkspaceTypedElement(location);
+	private static void compareLocalWithRef(final Repository repository,
+			final IPath location, final String refName,
+			final IWorkbenchPage page) {
 
-		CompareEditorInput in = prepareCompareInput(repository, gitPath, base,
-				refName);
+		Job job = new Job(UIText.CompareUtils_jobName) {
 
-		if (page != null)
-			openInCompare(page, in);
-		else
-			CompareUI.openCompareEditor(in);
+			@Override
+			public IStatus run(IProgressMonitor monitor) {
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
+				final String gitPath = getRepoRelativePath(location, repository);
+				final ITypedElement base = new LocalNonWorkspaceTypedElement(
+						location);
+
+				CompareEditorInput in;
+				try {
+					in = prepareCompareInput(repository, gitPath, base, refName);
+				} catch (IOException e) {
+					return Activator.createErrorStatus(
+							UIText.CompareWithRefAction_errorOnSynchronize, e);
+				}
+
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
+				openCompareEditorRunnable(page, in);
+				return Status.OK_STATUS;
+			}
+		};
+		job.setUser(true);
+		job.schedule();
 	}
 
 	/*
@@ -561,6 +661,65 @@ public class CompareUtils {
 					rightRev, includeLocal);
 	}
 
+	/**
+	 * This can be used to compare a given set of resources between two
+	 * revisions. If only one resource is to be compared, and that resource is
+	 * not part of a more important model (as defined in
+	 * {@link #canDirectlyOpenInCompare(IFile)}, we'll open a comparison editor
+	 * for that file alone, also taking leftPath and rightPath into account.
+	 * Otherwise, we'll launch a synchronization restrained of the given
+	 * resources set.
+	 * <p>
+	 * This can also be used to synchronize the whole repository if
+	 * <code>resources</code> is empty.
+	 * </p>
+	 * <p>
+	 * Note that this can be used to compare with the index by using
+	 * {@link GitFileRevision#INDEX} as either one of the two revs.
+	 * </p>
+	 *
+	 * @param resources
+	 *            The set of resources to compare. Can be empty (in which case
+	 *            we'll synchronize the whole repository).
+	 * @param repository
+	 *            The repository to load file revisions from.
+	 * @param leftPath
+	 *            The repository relative path to be used for the left revision,
+	 *            when comparing directly.
+	 * @param rightPath
+	 *            The repository relative path to be used for the right
+	 *            revision, when comparing directly.
+	 * @param leftRev
+	 *            Left revision of the comparison (usually the local or "new"
+	 *            revision). Won't be used if <code>includeLocal</code> is
+	 *            <code>true</code>.
+	 * @param rightRev
+	 *            Right revision of the comparison (usually the "old" revision).
+	 * @param includeLocal
+	 *            If <code>true</code>, this will use the local data as the
+	 *            "left" side of the comparison.
+	 * @param page
+	 *            If not {@null} try to re-use a compare editor on this
+	 *            page if any is available. Otherwise open a new one.
+	 * @throws IOException
+	 */
+	public static void compare(IResource[] resources, Repository repository,
+			String leftPath, String rightPath, String leftRev, String rightRev,
+			boolean includeLocal, IWorkbenchPage page) throws IOException {
+		if (resources.length == 1 && resources[0] instanceof IFile
+				&& canDirectlyOpenInCompare((IFile) resources[0])) {
+			if (includeLocal)
+				compareWorkspaceWithRef(repository, (IFile) resources[0],
+						rightRev, page);
+			else {
+				compareBetween(repository, leftPath, rightPath, leftRev,
+						rightRev, page);
+			}
+		} else
+			GitModelSynchronize.synchronize(resources, repository, leftRev,
+					rightRev, includeLocal);
+	}
+
 	/**
 	 * This can be used to compare a given file between two revisions.
 	 *
@@ -580,11 +739,10 @@ public class CompareUtils {
 	 * @param page
 	 *            If not {@null} try to re-use a compare editor on this
 	 *            page if any is available. Otherwise open a new one.
-	 * @throws IOException
 	 */
 	public static void compare(IPath location, Repository repository,
 			String leftRev, String rightRev, boolean includeLocal,
-			IWorkbenchPage page) throws IOException {
+			IWorkbenchPage page) {
 		if (includeLocal)
 			compareLocalWithRef(repository, location, rightRev, page);
 		else {
@@ -594,30 +752,76 @@ public class CompareUtils {
 	}
 
 	private static void compareBetween(Repository repository, String gitPath,
-			String leftRev, String rightRev, IWorkbenchPage page)
-			throws IOException {
-		final ITypedElement left = getTypedElementFor(repository, gitPath,
-				leftRev);
-		final ITypedElement right = getTypedElementFor(repository, gitPath,
-				rightRev);
+			String leftRev, String rightRev, IWorkbenchPage page) {
+		compareBetween(repository, gitPath, gitPath, leftRev, rightRev, page);
+	}
 
-		final ITypedElement commonAncestor;
-		if (left != null && right != null && !GitFileRevision.INDEX.equals(leftRev)
-				&& !GitFileRevision.INDEX.equals(rightRev))
-			commonAncestor = getTypedElementForCommonAncestor(repository,
-					gitPath, leftRev, rightRev);
-		else
-			commonAncestor = null;
+	/**
+	 * Compares two explicit files specified by leftGitPath and rightGitPath
+	 * between the two revisions leftRev and rightRev.
+	 *
+	 * @param repository
+	 *            The repository to load file revisions from.
+	 * @param leftGitPath
+	 *            The repository relative path to be used for the left revision.
+	 * @param rightGitPath
+	 *            The repository relative path to be used for the right
+	 *            revision.
+	 * @param leftRev
+	 *            Left revision of the comparison (usually the local or "new"
+	 *            revision). Won't be used if <code>includeLocal</code> is
+	 *            <code>true</code>.
+	 * @param rightRev
+	 *            Right revision of the comparison (usually the "old" revision).
+	 * @param page
+	 *            If not {@null} try to re-use a compare editor on this
+	 *            page if any is available. Otherwise open a new one.
+	 */
+	private static void compareBetween(final Repository repository,
+			final String leftGitPath, final String rightGitPath,
+			final String leftRev, final String rightRev,
+			final IWorkbenchPage page) {
 
-		final GitCompareFileRevisionEditorInput in = new GitCompareFileRevisionEditorInput(
-				left, right, commonAncestor, null);
-		in.getCompareConfiguration().setLeftLabel(leftRev);
-		in.getCompareConfiguration().setRightLabel(rightRev);
+		Job job = new Job(UIText.CompareUtils_jobName) {
 
-		if (page != null)
-			openInCompare(page, in);
-		else
-			CompareUI.openCompareEditor(in);
+			@Override
+			public IStatus run(IProgressMonitor monitor) {
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
+				final ITypedElement left;
+				final ITypedElement right;
+				try {
+					left = getTypedElementFor(repository, leftGitPath, leftRev);
+					right = getTypedElementFor(repository, rightGitPath,
+							rightRev);
+				} catch (IOException e) {
+					return Activator.createErrorStatus(
+							UIText.CompareWithRefAction_errorOnSynchronize, e);
+				}
+				final ITypedElement commonAncestor;
+				if (left != null && right != null
+						&& !GitFileRevision.INDEX.equals(leftRev)
+						&& !GitFileRevision.INDEX.equals(rightRev)) {
+					commonAncestor = getTypedElementForCommonAncestor(
+							repository, rightGitPath, leftRev, rightRev);
+				} else {
+					commonAncestor = null;
+				}
+
+				final GitCompareFileRevisionEditorInput in = new GitCompareFileRevisionEditorInput(
+						left, right, commonAncestor, null);
+				in.getCompareConfiguration().setLeftLabel(leftRev);
+				in.getCompareConfiguration().setRightLabel(rightRev);
+				if (monitor.isCanceled()) {
+					return Status.CANCEL_STATUS;
+				}
+				openCompareEditorRunnable(page, in);
+				return Status.OK_STATUS;
+			}
+		};
+		job.setUser(true);
+		job.schedule();
 	}
 
 	private static String getRepoRelativePath(IPath location,
