@@ -17,10 +17,11 @@ import java.util.List;
 
 import org.eclipse.core.expressions.PropertyTester;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.egit.core.internal.gerrit.GerritUtil;
 import org.eclipse.egit.core.project.RepositoryMapping;
-import org.eclipse.egit.ui.internal.gerrit.GerritUtil;
 import org.eclipse.egit.ui.internal.trace.GitTraceLocation;
 import org.eclipse.jgit.lib.Config;
+import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryState;
 import org.eclipse.jgit.transport.RefSpec;
@@ -162,7 +163,17 @@ public class ResourcePropertyTester extends PropertyTester {
 					String destination = pushSpec.getDestination();
 					if (destination == null)
 						continue;
-					return destination.startsWith(GerritUtil.REFS_FOR);
+					if (destination.startsWith(GerritUtil.REFS_FOR))
+						return true;
+				}
+				for (RefSpec fetchSpec : remoteConfig.getFetchRefSpecs()) {
+					String source = fetchSpec.getSource();
+					String destination = fetchSpec.getDestination();
+					if (source == null || destination == null)
+						continue;
+					if (source.startsWith(Constants.R_NOTES)
+							&& destination.startsWith(Constants.R_NOTES))
+						return true;
 				}
 			}
 		} catch (URISyntaxException e) {
