@@ -11,6 +11,8 @@
 
 package org.eclipse.egit.ui.internal.actions;
 
+import java.io.IOException;
+
 import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.resources.WorkspaceJob;
@@ -29,6 +31,8 @@ import org.eclipse.egit.ui.internal.dialogs.MergeTargetSelectionDialog;
 import org.eclipse.egit.ui.internal.merge.MergeResultDialog;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.MessageDialog;
+import org.eclipse.jgit.lib.Constants;
+import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryState;
 import org.eclipse.osgi.util.NLS;
@@ -46,7 +50,7 @@ public class MergeActionHandler extends RepositoryActionHandler {
 		if (repository == null)
 			return null;
 
-		if (!canMerge(repository, event))
+		if (!checkMergeIsPossible(repository, getShell(event)))
 			return null;
 		BasicConfigurationDialog.show(repository);
 		MergeTargetSelectionDialog mergeTargetSelectionDialog = new MergeTargetSelectionDialog(
@@ -118,4 +122,38 @@ public class MergeActionHandler extends RepositoryActionHandler {
 				&& repo.getRepositoryState() == RepositoryState.SAFE
 				&& isLocalBranchCheckedout(repo);
 	}
+
+	/**
+	 * Checks if merge is possible:
+	 * <ul>
+	 * <li>HEAD must point to a branch</li>
+	 * <li>Repository State must be SAFE</li>
+	 * </ul>
+	 * Shows an error dialog if a merge is not possible.
+	 *
+	 * @param repository
+	 *            the repository used for the merge
+	 * @param shell
+	 *            used to show a dialog in the error case
+	 * @return true if a merge is possible on the current HEAD
+	 */
+	public static boolean checkMergeIsPossible(Repository repository, Shell shell) {
+		String message = null;
+		try {
+			Ref head = repository.getRef(Constants.HEAD);
+			if (head == null || !head.isSymbolic())
+				message = UIText.MergeAction_HeadIsNoBranch;
+			else if (!repository.getRepositoryState().equals(
+					RepositoryState.SAFE))
+				message = NLS.bind(UIText.MergeAction_WrongRepositoryState,
+						repository.getRepositoryState());
+		} catch (IOException e) {
+			Activator.logError(e.getMessage(), e);
+			message = e.getMessage();
+		}
+
+		if (message != null)
+			MessageDialog.openError(shell, UIText.MergeAction_CannotMerge, message);
+		return (message == null);
+	}
 }
