@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Copyright (c) 2011, 2013 GitHub Inc and others.
+ *  Copyright (c) 2011, 2014 GitHub Inc and others.
  *  All rights reserved. This program and the accompanying materials
  *  are made available under the terms of the Eclipse Public License v1.0
  *  which accompanies this distribution, and is available at
@@ -7,6 +7,7 @@
  *
  *  Contributors:
  *    Kevin Sawicki (GitHub Inc.) - initial API and implementation
+ *    Marc-Andre Laperle (Ericsson) - Set the input to null when not visible
  *****************************************************************************/
 package org.eclipse.egit.ui.internal.blame;
 
@@ -98,6 +99,10 @@ public class BlameInformationControl extends AbstractInformationControl
 
 	private Composite diffComposite;
 
+	private Link showAnnotationsLink;
+
+	private SelectionAdapter showAnnotationsLinkSelectionAdapter;
+
 	/**
 	 * Create the information control for showing details on hover.
 	 *
@@ -221,6 +226,23 @@ public class BlameInformationControl extends AbstractInformationControl
 	}
 
 	public void setInput(Object input) {
+		if (input == null) {
+			// Make sure we don't hold a reference to this when nothing is
+			// shown, it can be big
+			this.revision = null;
+			if (showAnnotationsLink != null) {
+				// This listener can also hold a reference because of a final
+				// parameter, see createDiffLinkAndText(final RevCommit
+				// parent...)
+				if (!showAnnotationsLink.isDisposed())
+					showAnnotationsLink
+							.removeSelectionListener(showAnnotationsLinkSelectionAdapter);
+				showAnnotationsLink = null;
+				showAnnotationsLinkSelectionAdapter = null;
+			}
+			return;
+		}
+
 		this.revision = (BlameRevision) input;
 
 		// Remember line number that was hovered over when the input was set.
@@ -322,17 +344,20 @@ public class BlameInformationControl extends AbstractInformationControl
 				UIText.BlameInformationControl_DiffHeaderLabel, parentId,
 				parentMessage));
 
-		Link showAnnotationsLink = new Link(header, SWT.NONE);
+		showAnnotationsLink = new Link(header, SWT.NONE);
 		showAnnotationsLink
 				.setText(UIText.BlameInformationControl_ShowAnnotationsLink);
-		showAnnotationsLink.addSelectionListener(new SelectionAdapter() {
+		showAnnotationsLinkSelectionAdapter = new SelectionAdapter() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
 				blameParent(parent, diff, parentLine);
 			}
-		});
+		};
+		showAnnotationsLink
+				.addSelectionListener(showAnnotationsLinkSelectionAdapter);
 
-		DiffViewer diffText = new DiffViewer(diffComposite, null, SWT.NONE);
+		DiffViewer diffText = new DiffViewer(diffComposite, null, SWT.NONE,
+				false);
 		diffText.setEditable(false);
 		diffText.getControl().setLayoutData(
 				GridDataFactory.fillDefaults().grab(true, true).create());
@@ -391,9 +416,16 @@ public class BlameInformationControl extends AbstractInformationControl
 
 	private void showCommitInHistory() {
 		getShell().dispose();
-		IHistoryView part = (IHistoryView) PlatformUI.getWorkbench()
-				.getActiveWorkbenchWindow().getActivePage()
-				.findView(IHistoryView.VIEW_ID);
+		IHistoryView part;
+		try {
+			part = (IHistoryView) PlatformUI.getWorkbench()
+					.getActiveWorkbenchWindow().getActivePage()
+					.showView(IHistoryView.VIEW_ID);
+		} catch (PartInitException e) {
+			Activator.logError(e.getLocalizedMessage(), e);
+			return;
+		}
+
 		if (part == null)
 			return;
 
@@ -431,4 +463,10 @@ public class BlameInformationControl extends AbstractInformationControl
 			Activator.logError(UIText.ShowBlameHandler_errorMessage, e);
 		}
 	}
+
+	public void setVisible(boolean visible) {
+		super.setVisible(visible);
+		if (!visible)
+			setInput(null);
+	}
 }
