@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 SAP AG.
+ * Copyright (c) 2010, 2013 SAP AG.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,8 +10,12 @@
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.commands.shared;
 
+import org.eclipse.core.commands.ExecutionException;
+import org.eclipse.egit.core.op.RebaseOperation;
 import org.eclipse.egit.ui.internal.UIText;
+import org.eclipse.egit.ui.internal.rebase.RebaseInteractiveHandler;
 import org.eclipse.jgit.api.RebaseCommand.Operation;
+import org.eclipse.jgit.lib.Repository;
 
 /**
  * Implements "Continue Rebase"
@@ -21,7 +25,14 @@ public class ContinueRebaseCommand extends AbstractRebaseCommandHandler {
 	 * Default constructor
 	 */
 	public ContinueRebaseCommand() {
-		super(Operation.CONTINUE, UIText.ContinueRebaseCommand_JobName,
+		super(UIText.ContinueRebaseCommand_JobName,
 				UIText.ContinueRebaseCommand_CancelDialogMessage);
 	}
+
+	@Override
+	protected RebaseOperation createRebaseOperation(Repository repository)
+			throws ExecutionException {
+		return new RebaseOperation(repository, Operation.CONTINUE,
+				RebaseInteractiveHandler.INSTANCE);
+	}
 }
