@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010, 2013 SAP AG and others.
+ * Copyright (c) 2010, 2014 SAP AG and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -23,9 +23,12 @@ import org.eclipse.egit.core.op.RebaseOperation;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.dialogs.BasicConfigurationDialog;
 import org.eclipse.egit.ui.internal.dialogs.RebaseTargetSelectionDialog;
+import org.eclipse.egit.ui.internal.rebase.RebaseInteractiveHandler;
+import org.eclipse.egit.ui.internal.selection.SelectionUtils;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.jgit.api.RebaseCommand.InteractiveHandler;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
@@ -39,6 +42,10 @@ public class RebaseCurrentRefCommand extends AbstractRebaseCommandHandler {
 
 	private Ref ref;
 
+	private boolean interactive;
+
+	private boolean preserveMerges;
+
 	/** */
 	public RebaseCurrentRefCommand() {
 		super(UIText.RebaseCurrentRefCommand_RebasingCurrentJobName,
@@ -63,7 +70,12 @@ public class RebaseCurrentRefCommand extends AbstractRebaseCommandHandler {
 		} else
 			ref = null;
 
-		final Repository repository = getRepository(event);
+		Object context = event.getApplicationContext();
+		if (!(context instanceof IEvaluationContext))
+			return;
+
+		final Repository repository = SelectionUtils
+				.getRepository((IEvaluationContext) context);
 		if (repository == null)
 			return;
 
@@ -83,6 +95,8 @@ public class RebaseCurrentRefCommand extends AbstractRebaseCommandHandler {
 				} catch (IOException e) {
 					throw new ExecutionException(e.getMessage(), e);
 				}
+				interactive = rebaseTargetSelectionDialog.isInteractive();
+				preserveMerges = rebaseTargetSelectionDialog.isPreserveMerges();
 			} else
 				return;
 		}
@@ -96,17 +110,15 @@ public class RebaseCurrentRefCommand extends AbstractRebaseCommandHandler {
 	public void setEnabled(Object evaluationContext) {
 		if (evaluationContext instanceof IEvaluationContext) {
 			IEvaluationContext ctx = (IEvaluationContext) evaluationContext;
-			Object selection = getSelection(ctx);
-			if (selection instanceof ISelection) {
-				Repository repo = getRepository((ISelection) selection, getActiveEditorInput(ctx));
-				if (repo != null) {
-					boolean enabled = isEnabledForState(repo,
-							repo.getRepositoryState());
-					setBaseEnabled(enabled);
-				} else
-					setBaseEnabled(false);
-				return;
+			Repository repo = SelectionUtils.getRepository(ctx);
+			if (repo != null) {
+				boolean enabled = isEnabledForState(repo,
+						repo.getRepositoryState());
+				setBaseEnabled(enabled);
+			} else {
+				setBaseEnabled(false);
 			}
+			return;
 		}
 		setBaseEnabled(true);
 	}
@@ -144,6 +156,11 @@ public class RebaseCurrentRefCommand extends AbstractRebaseCommandHandler {
 	@Override
 	protected RebaseOperation createRebaseOperation(Repository repository)
 			throws ExecutionException {
-		return new RebaseOperation(repository, ref);
+		InteractiveHandler handler = interactive ? RebaseInteractiveHandler.INSTANCE
+				: null;
+		RebaseOperation operation = new RebaseOperation(repository, ref,
+				handler);
+		operation.setPreserveMerges(preserveMerges);
+		return operation;
 	}
 }
