@@ -57,6 +57,7 @@ import org.eclipse.swt.custom.CLabel;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.widgets.Composite;
@@ -140,7 +141,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		return (Image) this.resources.get(descriptor);
 	}
 
-	private Section createSection(Composite parent, FormToolkit toolkit,
+	Section createSection(Composite parent, FormToolkit toolkit,
 			int span) {
 		Section section = toolkit.createSection(parent,
 				ExpandableComposite.TITLE_BAR | ExpandableComposite.TWISTIE
@@ -150,7 +151,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		return section;
 	}
 
-	private Composite createSectionClient(Section parent, FormToolkit toolkit) {
+	Composite createSectionClient(Section parent, FormToolkit toolkit) {
 		Composite client = toolkit.createComposite(parent);
 		GridLayoutFactory.fillDefaults().extendedMargins(2, 2, 2, 2)
 				.applyTo(client);
@@ -214,7 +215,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		return userArea;
 	}
 
-	private void updateSectionClient(Section section, Composite client,
+	void updateSectionClient(Section section, Composite client,
 			FormToolkit toolkit) {
 		hookExpansionGrabbing(section);
 		toolkit.paintBordersFor(client);
@@ -243,41 +244,49 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 			createUserArea(userArea, toolkit, committer, false);
 
 		int count = commit.getParentCount();
-		if (count > 0) {
-			Composite parents = toolkit.createComposite(top);
-			GridLayoutFactory.fillDefaults().spacing(2, 2).numColumns(2)
-					.applyTo(parents);
-			GridDataFactory.fillDefaults().grab(false, false).applyTo(parents);
-
-			for (int i = 0; i < count; i++) {
-				final RevCommit parentCommit = commit.getParent(i);
-				toolkit.createLabel(parents,
-						UIText.CommitEditorPage_LabelParent).setForeground(
-						toolkit.getColors().getColor(IFormColors.TB_TOGGLE));
-				final Hyperlink link = toolkit
-						.createHyperlink(parents,
-								parentCommit.abbreviate(PARENT_LENGTH).name(),
-								SWT.NONE);
-				link.addHyperlinkListener(new HyperlinkAdapter() {
-
-					public void linkActivated(HyperlinkEvent e) {
-						try {
-							CommitEditor.open(new RepositoryCommit(getCommit()
-									.getRepository(), parentCommit));
-							if ((e.getStateMask() & SWT.MOD1) != 0)
-								getEditor().close(false);
-						} catch (PartInitException e1) {
-							Activator.logError(
-									"Error opening commit editor", e1);//$NON-NLS-1$
-						}
-					}
-				});
-			}
-		}
+		if (count > 0)
+			createParentsArea(top, toolkit, commit);
 
 		createTagsArea(userArea, toolkit, 2);
 	}
 
+	private void createParentsArea(Composite parent, FormToolkit toolkit,
+			RevCommit commit) {
+		Composite parents = toolkit.createComposite(parent);
+		GridLayoutFactory.fillDefaults().spacing(2, 2).numColumns(2)
+				.applyTo(parents);
+		GridDataFactory.fillDefaults().grab(false, false).applyTo(parents);
+
+		for (int i = 0; i < commit.getParentCount(); i++) {
+			final RevCommit parentCommit = commit.getParent(i);
+			toolkit.createLabel(parents, getParentCommitLabel(i))
+					.setForeground(
+							toolkit.getColors().getColor(IFormColors.TB_TOGGLE));
+			final Hyperlink link = toolkit
+					.createHyperlink(parents,
+							parentCommit.abbreviate(PARENT_LENGTH).name(),
+							SWT.NONE);
+			link.addHyperlinkListener(new HyperlinkAdapter() {
+				public void linkActivated(HyperlinkEvent e) {
+					try {
+						CommitEditor.open(new RepositoryCommit(getCommit()
+								.getRepository(), parentCommit));
+						if ((e.getStateMask() & SWT.MOD1) != 0)
+							getEditor().close(false);
+					} catch (PartInitException e1) {
+						Activator.logError(
+								"Error opening commit editor", e1);//$NON-NLS-1$
+					}
+				}
+			});
+		}
+	}
+
+	@SuppressWarnings("unused")
+	String getParentCommitLabel(int i) {
+		return UIText.CommitEditorPage_LabelParent;
+	}
+
 	private List<Ref> getTags() {
 		Repository repository = getCommit().getRepository();
 		List<Ref> tags = new ArrayList<Ref>(repository.getTags().values());
@@ -292,7 +301,8 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		return tags;
 	}
 
-	private void createTagsArea(Composite parent, FormToolkit toolkit, int span) {
+	void createTagsArea(Composite parent, FormToolkit toolkit,
+			int span) {
 		Composite tagArea = toolkit.createComposite(parent);
 		GridLayoutFactory.fillDefaults().numColumns(2).equalWidth(false)
 				.applyTo(tagArea);
@@ -306,17 +316,30 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		GridLayoutFactory.fillDefaults().spacing(1, 1).applyTo(tagLabelArea);
 	}
 
-	private void fillDiffs(FileDiff[] diffs) {
+	void fillDiffs(FileDiff[] diffs) {
 		diffViewer.setInput(diffs);
-		diffSection.setText(MessageFormat.format(
-				UIText.CommitEditorPage_SectionFiles,
-				Integer.valueOf(diffs.length)));
+		diffSection.setText(getDiffSectionTitle(Integer.valueOf(diffs.length)));
+		setSectionExpanded(diffSection, diffs.length != 0);
 	}
 
-	private void fillTags(FormToolkit toolkit, List<Ref> tags) {
+	static void setSectionExpanded(Section section, boolean expanded) {
+		section.setExpanded(expanded);
+		((GridData) section.getLayoutData()).grabExcessVerticalSpace = expanded;
+	}
+
+	String getDiffSectionTitle(Integer numChanges) {
+		return MessageFormat.format(UIText.CommitEditorPage_SectionFiles,
+				numChanges);
+	}
+
+	void fillTags(FormToolkit toolkit, List<Ref> tags) {
 		for (Control child : tagLabelArea.getChildren())
 			child.dispose();
 
+		// Hide "Tags" area if no tags to show
+		((GridData) tagLabelArea.getParent().getLayoutData()).exclude = tags
+				.isEmpty();
+
 		GridLayoutFactory.fillDefaults().spacing(1, 1).numColumns(tags.size())
 				.applyTo(tagLabelArea);
 
@@ -370,11 +393,16 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 			}
 
 		};
+
 		if ((toolkit.getBorderStyle() & SWT.BORDER) == 0)
 			textContent.setData(FormToolkit.KEY_DRAW_BORDER,
 					FormToolkit.TEXT_BORDER);
-		GridDataFactory.fillDefaults().hint(SWT.DEFAULT, 80).grab(true, true)
-				.applyTo(textContent);
+
+		Point size = textContent.getTextWidget().computeSize(SWT.DEFAULT,
+				SWT.DEFAULT);
+		int yHint = size.y > 80 ? 80 : SWT.DEFAULT;
+		GridDataFactory.fillDefaults().hint(SWT.DEFAULT, yHint).minSize(1, 20)
+				.grab(true, true).applyTo(textContent);
 
 		updateSectionClient(messageSection, messageArea, toolkit);
 	}
@@ -411,7 +439,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 				Integer.valueOf(result.size())));
 	}
 
-	private void createFilesArea(Composite parent, FormToolkit toolkit, int span) {
+	void createDiffArea(Composite parent, FormToolkit toolkit, int span) {
 		diffSection = createSection(parent, toolkit, span);
 		diffSection.setText(UIText.CommitEditorPage_SectionFilesEmpty);
 		Composite filesArea = createSectionClient(diffSection, toolkit);
@@ -421,7 +449,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 				| toolkit.getBorderStyle());
 		diffViewer.getTable().setData(FormToolkit.KEY_DRAW_BORDER,
 				FormToolkit.TREE_BORDER);
-		GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 80)
+		GridDataFactory.fillDefaults().grab(true, true)
 				.applyTo(diffViewer.getControl());
 		diffViewer.setContentProvider(ArrayContentProvider.getInstance());
 		diffViewer.setTreeWalk(getCommit().getRepository(), null);
@@ -429,7 +457,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		updateSectionClient(diffSection, filesArea, toolkit);
 	}
 
-	private RepositoryCommit getCommit() {
+	RepositoryCommit getCommit() {
 		return (RepositoryCommit) getEditor()
 				.getAdapter(RepositoryCommit.class);
 	}
@@ -457,12 +485,16 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 
 		createHeaderArea(displayArea, toolkit, 2);
 		createMessageArea(displayArea, toolkit, 2);
-		createFilesArea(displayArea, toolkit, 1);
-		createBranchesArea(displayArea, toolkit, 1);
+		createChangesArea(displayArea, toolkit);
 
 		loadSections();
 	}
 
+	void createChangesArea(Composite displayArea, FormToolkit toolkit) {
+		createDiffArea(displayArea, toolkit, 1);
+		createBranchesArea(displayArea, toolkit, 1);
+	}
+
 	private List<Ref> loadTags() {
 		RepositoryCommit repoCommit = getCommit();
 		RevCommit commit = repoCommit.getRevCommit();
@@ -497,7 +529,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		}
 	}
 
-	private void loadSections() {
+	void loadSections() {
 		RepositoryCommit commit = getCommit();
 		Job refreshJob = new Job(MessageFormat.format(
 				UIText.CommitEditorPage_JobName, commit.getRevCommit().name())) {
@@ -519,6 +551,7 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 							fillTags(getManagedForm().getToolkit(), tags);
 							fillDiffs(diffs);
 							fillBranches(branches);
+							form.reflow(true);
 							form.layout(true, true);
 						}
 					});
