@@ -115,15 +115,19 @@ public class DiffViewer extends SourceViewer {
 	 * @param parent
 	 * @param ruler
 	 * @param styles
+	 * @param showCursorLine
 	 */
-	public DiffViewer(Composite parent, IVerticalRuler ruler, int styles) {
+	public DiffViewer(Composite parent, IVerticalRuler ruler, int styles,
+			boolean showCursorLine) {
 		super(parent, ruler, styles);
 		setDocument(new Document());
 		SourceViewerDecorationSupport support = new SourceViewerDecorationSupport(
 				this, null, null, EditorsUI.getSharedTextColors());
-		support.setCursorLinePainterPreferenceKeys(
-				AbstractDecoratedTextEditorPreferenceConstants.EDITOR_CURRENT_LINE,
-				AbstractDecoratedTextEditorPreferenceConstants.EDITOR_CURRENT_LINE_COLOR);
+		if (showCursorLine) {
+			support.setCursorLinePainterPreferenceKeys(
+					AbstractDecoratedTextEditorPreferenceConstants.EDITOR_CURRENT_LINE,
+					AbstractDecoratedTextEditorPreferenceConstants.EDITOR_CURRENT_LINE_COLOR);
+		}
 		support.install(EditorsUI.getPreferenceStore());
 		if (ruler instanceof CompositeRuler) {
 			lineNumberRuler = new LineNumberRulerColumn();
