@@ -1,6 +1,7 @@
 /******************************************************************************
  *  Copyright (c) 2010 SAP AG.
- *  Copyright (c) 2011 GitHub Inc.
+ *  Copyright (c) 2011, 2014 GitHub Inc.
+ *  and other copyright owners as documented in the project's IP log.
  *  All rights reserved. This program and the accompanying materials
  *  are made available under the terms of the Eclipse Public License v1.0
  *  which accompanies this distribution, and is available at
@@ -13,13 +14,15 @@ package org.eclipse.egit.ui.internal.commit.command;
 
 import java.io.IOException;
 import java.text.MessageFormat;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
-import org.eclipse.core.resources.WorkspaceJob;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
@@ -29,17 +32,31 @@ import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.egit.core.op.CherryPickOperation;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.JobFamilies;
+import org.eclipse.egit.ui.internal.UIRepositoryUtils;
 import org.eclipse.egit.ui.internal.UIText;
+import org.eclipse.egit.ui.internal.commit.RepositoryCommit;
 import org.eclipse.egit.ui.internal.handler.SelectionHandler;
+import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.MessageDialog;
+import org.eclipse.jface.layout.GridDataFactory;
+import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider;
+import org.eclipse.jface.viewers.TreeViewer;
+import org.eclipse.jface.window.Window;
 import org.eclipse.jgit.api.CherryPickResult;
 import org.eclipse.jgit.api.CherryPickResult.CherryPickStatus;
+import org.eclipse.jgit.api.errors.GitAPIException;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.merge.ResolveMerger.MergeFailureReason;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.osgi.util.NLS;
+import org.eclipse.swt.SWT;
+import org.eclipse.swt.layout.FillLayout;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.model.WorkbenchContentProvider;
+import org.eclipse.ui.model.WorkbenchLabelProvider;
 
 /**
  * Handler to cherry pick the commit onto the current branch
@@ -63,13 +80,19 @@ public class CherryPickHandler extends SelectionHandler {
 		if (!confirmCherryPick(parent, repo, commit))
 			return null;
 
+		try {
+			if (!UIRepositoryUtils.handleUncommittedFiles(repo, parent))
+				return null;
+		} catch (GitAPIException e) {
+			Activator.logError(e.getMessage(), e);
+			return null;
+		}
+
 		final CherryPickOperation op = new CherryPickOperation(repo, commit);
-
-		Job job = new WorkspaceJob(MessageFormat.format(
-				UIText.CherryPickHandler_JobName, commit.name())) {
-
+		Job job = new Job(MessageFormat.format(
+				UIText.CherryPickHandler_JobName, 1)) {
 			@Override
-			public IStatus runInWorkspace(IProgressMonitor monitor) {
+			protected IStatus run(IProgressMonitor monitor) {
 				try {
 					op.execute(monitor);
 					CherryPickResult cherryPickResult = op.getResult();
@@ -117,8 +140,8 @@ public class CherryPickHandler extends SelectionHandler {
 		final String message;
 		try {
 			message = MessageFormat.format(
-					UIText.CherryPickHandler_ConfirmMessage,
-					commit.abbreviate(7).name(), repository.getBranch());
+					UIText.CherryPickHandler_ConfirmMessage, 1,
+					repository.getBranch());
 		} catch (IOException e) {
 			throw new ExecutionException(
 					"Exception obtaining current repository branch", e); //$NON-NLS-1$
@@ -127,13 +150,63 @@ public class CherryPickHandler extends SelectionHandler {
 		shell.getDisplay().syncExec(new Runnable() {
 
 			public void run() {
-				confirmed.set(MessageDialog.openConfirm(shell,
-						UIText.CherryPickHandler_ConfirmTitle, message));
+				ConfirmCherryPickDialog dialog = new ConfirmCherryPickDialog(
+						shell, message, repository, Arrays.asList(commit));
+				int result = dialog.open();
+				confirmed.set(result == Window.OK);
 			}
 		});
 		return confirmed.get();
 	}
 
+	private static class ConfirmCherryPickDialog extends MessageDialog {
+
+		private RepositoryCommit[] commits;
+
+		public ConfirmCherryPickDialog(Shell parentShell,
+				String message, Repository repository, List<RevCommit> revCommits) {
+			super(parentShell, UIText.CherryPickHandler_ConfirmTitle, null,
+					message, MessageDialog.CONFIRM, new String[] {
+							IDialogConstants.OK_LABEL,
+							IDialogConstants.CANCEL_LABEL }, 0);
+			setShellStyle(getShellStyle() | SWT.RESIZE);
+
+			List<RepositoryCommit> repoCommits = new ArrayList<RepositoryCommit>();
+			for (RevCommit commit : revCommits)
+				repoCommits.add(new RepositoryCommit(repository, commit));
+			this.commits = repoCommits.toArray(new RepositoryCommit[0]);
+		}
+
+		@Override
+		protected Control createCustomArea(Composite parent) {
+			Composite area = new Composite(parent, SWT.NONE);
+			area.setLayoutData(GridDataFactory.fillDefaults().grab(true, true)
+					.create());
+			area.setLayout(new FillLayout());
+
+			TreeViewer treeViewer = new TreeViewer(area);
+			treeViewer.setContentProvider(new ContentProvider());
+			treeViewer.setLabelProvider(new DelegatingStyledCellLabelProvider(
+					new WorkbenchLabelProvider()));
+			treeViewer.setInput(commits);
+
+			return area;
+		}
+
+		private static class ContentProvider extends WorkbenchContentProvider {
+
+			public Object[] getElements(final Object element) {
+				return (Object[]) element;
+			}
+
+			public Object[] getChildren(Object element) {
+				if (element instanceof RepositoryCommit)
+					return ((RepositoryCommit) element).getDiffs();
+				return super.getChildren(element);
+			}
+		}
+	}
+
 	private void showNotPerformedDialog(final Shell shell) {
 		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
 
