@@ -602,8 +602,9 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 			}
 		});
 
-		newLabel(g, UIText.RepositorySelectionPage_storeInSecureStore);
 		storeCheckbox = new Button(g, SWT.CHECK);
+		storeCheckbox
+				.setText(UIText.RepositorySelectionPage_storeInSecureStore);
 		storeCheckbox.setSelection(storeInSecureStore);
 		storeCheckbox.addSelectionListener(new SelectionListener() {
 			public void widgetSelected(SelectionEvent e) {
@@ -636,7 +637,7 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 					setURI(uri.setScheme(nullString(scheme.getItem(idx))));
 					scheme.setToolTipText(Protocol.values()[idx].getTooltip());
 				}
-				updateAuthGroup();
+				updateGroups();
 			}
 		});
 
@@ -824,7 +825,7 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 						String p = credentials.getPassword();
 						String uriUser = finalURI.getUser();
 						if (uriUser == null) {
-							if (setSafeUser(u) || setSafePassword(p))
+							if (setSafeUser(u) && setSafePassword(p))
 								setStoreInSecureStore(true);
 						} else if (uriUser.length() != 0 && uriUser.equals(u)) {
 							if (setSafePassword(p))
@@ -904,17 +905,27 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 	private void updateRemoteAndURIPanels() {
 		UIUtils.setEnabledRecursively(uriPanel, isURISelected());
 		if (uriPanel.getEnabled())
-			updateAuthGroup();
+			updateGroups();
 		if (configuredRemotes != null)
 			UIUtils.setEnabledRecursively(remotePanel, !isURISelected());
 	}
 
-	private void updateAuthGroup() {
+	private void updateGroups() {
 		Protocol p = getProtocol();
 		if (p != null) {
 			hostText.setEnabled(p.hasHost());
+			if (!p.hasHost())
+				hostText.setText(EMPTY_STRING);
 			portText.setEnabled(p.hasPort());
+			if (!p.hasPort())
+				portText.setText(EMPTY_STRING);
+
 			UIUtils.setEnabledRecursively(authGroup, p.canAuthenticate());
+			if (!p.canAuthenticate()) {
+				userText.setText(EMPTY_STRING);
+				passText.setText(EMPTY_STRING);
+				storeCheckbox.setSelection(false);
+			}
 		}
 	}
 
@@ -993,7 +1004,7 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 				scheme.notifyListeners(SWT.Selection, new Event());
 			}
 
-			updateAuthGroup();
+			updateGroups();
 			uri = u;
 		} catch (URISyntaxException err) {
 			// leave uriText as it is, but clean up underlying uri and
