@@ -15,9 +15,9 @@ import java.util.concurrent.atomic.AtomicReference;
 
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.securestorage.UserPasswordCredentials;
+import org.eclipse.egit.ui.internal.SecureStoreUtils;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.dialogs.CustomPromptDialog;
-import org.eclipse.equinox.security.storage.StorageException;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.window.Window;
@@ -102,7 +102,7 @@ public class EGitCredentialsProvider extends CredentialsProvider {
 			if ((user != null) && (password != null))
 				credentials = new UserPasswordCredentials(user, password);
 			else
-				credentials = getCredentialsFromSecureStore(uri);
+				credentials = SecureStoreUtils.getCredentialsQuietly(uri);
 
 			if (credentials == null) {
 				credentials = getCredentialsFromUser(uri);
@@ -229,16 +229,4 @@ public class EGitCredentialsProvider extends CredentialsProvider {
 		});
 		return aRef.get();
 	}
-
-	private UserPasswordCredentials getCredentialsFromSecureStore(final URIish uri) {
-		UserPasswordCredentials credentials = null;
-		try {
-			credentials = Activator.getDefault().getSecureStore()
-					.getCredentials(uri);
-		} catch (StorageException e) {
-			Activator.logError(
-					UIText.EGitCredentialsProvider_errorReadingCredentials, e);
-		}
-		return credentials;
-	}
 }
