@@ -17,7 +17,9 @@
 package org.eclipse.egit.ui.internal.decorators;
 
 import java.io.IOException;
+import java.util.Arrays;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
 import org.eclipse.core.resources.IResource;
@@ -81,19 +83,19 @@ public class GitLightweightDecorator extends LabelProvider implements
 	/**
 	 * Collector for keeping the error view from filling up with exceptions
 	 */
-	private static ExceptionCollector exceptions = new ExceptionCollector(
+	private static final ExceptionCollector EXCEPTION_COLLECTOR = new ExceptionCollector(
 			UIText.Decorator_exceptionMessage, Activator.getPluginId(),
 			IStatus.ERROR, Activator.getDefault().getLog());
 
-	private static String[] fonts = new String[]  {
+	private static final List<String> FONT_IDS = Arrays.asList(
 			UIPreferences.THEME_UncommittedChangeFont,
-			UIPreferences.THEME_IgnoredResourceFont };
+			UIPreferences.THEME_IgnoredResourceFont);
 
-	private static String[] colors = new String[] {
+	private static final List<String> COLOR_IDS = Arrays.asList(
 		UIPreferences.THEME_UncommittedChangeBackgroundColor,
 			UIPreferences.THEME_UncommittedChangeForegroundColor,
 			UIPreferences.THEME_IgnoredResourceBackgroundColor,
-			UIPreferences.THEME_IgnoredResourceForegroundColor };
+			UIPreferences.THEME_IgnoredResourceForegroundColor);
 
 	private static RGB defaultBackgroundRgb;
 
@@ -101,15 +103,15 @@ public class GitLightweightDecorator extends LabelProvider implements
 	 * Constructs a new Git resource decorator
 	 */
 	public GitLightweightDecorator() {
+		// This is an optimization to ensure that while decorating our fonts and
+		// colors are pre-created and decoration can occur without having to syncExec.
+		ensureFontAndColorsCreated(FONT_IDS, COLOR_IDS);
 		TeamUI.addPropertyChangeListener(this);
 		Activator.addPropertyChangeListener(this);
 		PlatformUI.getWorkbench().getThemeManager().getCurrentTheme()
 				.addPropertyChangeListener(this);
 
 		org.eclipse.egit.core.Activator.getDefault().getIndexDiffCache().addIndexDiffChangedListener(this);
-		// This is an optimization to ensure that while decorating our fonts and colors are
-		// pre-created and decoration can occur without having to syncExec.
-		ensureFontAndColorsCreated(fonts, colors);
 	}
 
 	/**
@@ -120,17 +122,18 @@ public class GitLightweightDecorator extends LabelProvider implements
 	 * @param actFonts fonts ids to cache
 	 * @param actColors color ids to cache
 	 */
-	private void ensureFontAndColorsCreated(final String[] actFonts, final String[] actColors) {
-		final Display display = Display.getDefault();
+	private void ensureFontAndColorsCreated(final List<String> actFonts,
+			final List<String> actColors) {
+		final Display display = PlatformUI.getWorkbench().getDisplay();
 		display.syncExec(new Runnable() {
 			public void run() {
 				ITheme theme  = PlatformUI.getWorkbench().getThemeManager().getCurrentTheme();
-				for (int i = 0; i < actColors.length; i++) {
-					theme.getColorRegistry().get(actColors[i]);
+				for (int i = 0; i < actColors.size(); i++) {
+					theme.getColorRegistry().get(actColors.get(i));
 
 				}
-				for (int i = 0; i < actFonts.length; i++) {
-					theme.getFontRegistry().get(actFonts[i]);
+				for (int i = 0; i < actFonts.size(); i++) {
+					theme.getFontRegistry().get(actFonts.get(i));
 				}
 				defaultBackgroundRgb = display.getSystemColor(
 						SWT.COLOR_LIST_BACKGROUND).getRGB();
@@ -138,11 +141,6 @@ public class GitLightweightDecorator extends LabelProvider implements
 		});
 	}
 
-	/*
-	 * (non-Javadoc)
-	 *
-	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
-	 */
 	@Override
 	public void dispose() {
 		super.dispose();
@@ -385,26 +383,33 @@ public class GitLightweightDecorator extends LabelProvider implements
 		private void decorateFontAndColour(IDecoration decoration,
 				IDecoratableResource resource) {
 			ITheme current = PlatformUI.getWorkbench().getThemeManager().getCurrentTheme();
+			Color bc = null;
+			Color fc = null;
+			Font f = null;
 			if (resource.isIgnored()) {
-				Color bc = current.getColorRegistry().get(
+				bc = current.getColorRegistry().get(
 						UIPreferences.THEME_IgnoredResourceBackgroundColor);
-				Color fc = current.getColorRegistry().get(
+				fc = current.getColorRegistry().get(
 						UIPreferences.THEME_IgnoredResourceForegroundColor);
-				Font f = current.getFontRegistry().get(
+				f = current.getFontRegistry().get(
 						UIPreferences.THEME_IgnoredResourceFont);
-
-				setBackgroundColor(decoration, bc);
-				decoration.setForegroundColor(fc);
-				decoration.setFont(f);
 			} else if (!resource.isTracked()
 					|| resource.isDirty()
 					|| resource.staged() != Staged.NOT_STAGED) {
-				Color bc = current.getColorRegistry().get(UIPreferences.THEME_UncommittedChangeBackgroundColor);
-				Color fc = current.getColorRegistry().get(UIPreferences.THEME_UncommittedChangeForegroundColor);
-				Font f = current.getFontRegistry().get(UIPreferences.THEME_UncommittedChangeFont);
-
+				bc = current.getColorRegistry().get(
+						UIPreferences.THEME_UncommittedChangeBackgroundColor);
+				fc = current.getColorRegistry().get(
+						UIPreferences.THEME_UncommittedChangeForegroundColor);
+				f = current.getFontRegistry().get(
+						UIPreferences.THEME_UncommittedChangeFont);
+			}
+			if (bc != null) {
 				setBackgroundColor(decoration, bc);
+			}
+			if (fc != null) {
 				decoration.setForegroundColor(fc);
+			}
+			if (f != null) {
 				decoration.setFont(f);
 			}
 		}
@@ -609,7 +614,7 @@ public class GitLightweightDecorator extends LabelProvider implements
 	 * Perform a blanket refresh of all decorations
 	 */
 	public static void refresh() {
-		Display.getDefault().asyncExec(new Runnable() {
+		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
 			public void run() {
 				Activator.getDefault().getWorkbench().getDecoratorManager()
 						.update(DECORATOR_ID);
@@ -638,13 +643,15 @@ public class GitLightweightDecorator extends LabelProvider implements
 				|| prop.equals(UIPreferences.THEME_IgnoredResourceFont)
 				|| prop.equals(UIPreferences.THEME_IgnoredResourceBackgroundColor)
 				|| prop.equals(UIPreferences.THEME_IgnoredResourceForegroundColor)) {
-			ensureFontAndColorsCreated(fonts, colors);
+			ensureFontAndColorsCreated(FONT_IDS, COLOR_IDS);
 			postLabelEvent(); // TODO do I really need this?
 		}
 	}
 
 	public void indexDiffChanged(Repository repository,
 			IndexDiffData indexDiffData) {
+		// clear calculated repo data
+		DecoratableResourceHelper.clearState(repository);
 		postLabelEvent();
 	}
 
@@ -689,7 +696,7 @@ public class GitLightweightDecorator extends LabelProvider implements
 		final LabelProviderChangedEvent event = new LabelProviderChangedEvent(
 				this);
 		// Re-trigger decoration process (in UI thread)
-		Display.getDefault().asyncExec(new Runnable() {
+		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
 			public void run() {
 				fireLabelProviderChanged(event);
 			}
@@ -707,7 +714,7 @@ public class GitLightweightDecorator extends LabelProvider implements
 	 */
 	private static void handleException(IResource resource, CoreException e) {
 		if (resource == null || resource.isAccessible())
-			exceptions.handleException(e);
+			EXCEPTION_COLLECTOR.handleException(e);
 	}
 }
 
@@ -738,7 +745,7 @@ class LabelEventJob extends Job {
 		super(name);
 	}
 
-	private GitLightweightDecorator glwDecorator = null;
+	private GitLightweightDecorator glwDecorator;
 
 	/**
 	 * Post a label event
@@ -748,8 +755,8 @@ class LabelEventJob extends Job {
 	 *            LabelProviderChangedEvent
 	 */
 	void postLabelEvent(final GitLightweightDecorator decorator) {
-		if (this.glwDecorator == null)
-			this.glwDecorator = decorator;
+		if (glwDecorator == null)
+			glwDecorator = decorator;
 		if (getState() == SLEEPING || getState() == WAITING)
 			cancel();
 		schedule(DELAY);
