@@ -37,7 +37,7 @@ import org.eclipse.egit.core.op.CreatePatchOperation.DiffHeaderFormat;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIUtils;
-import org.eclipse.egit.ui.internal.GitLabelProvider;
+import org.eclipse.egit.ui.internal.CommonUtils;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.actions.ResetMenu;
 import org.eclipse.egit.ui.internal.history.SWTCommitList.SWTLane;
@@ -48,10 +48,6 @@ import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
 import org.eclipse.jface.resource.ResourceManager;
-import org.eclipse.jface.text.AbstractHoverInformationControlManager;
-import org.eclipse.jface.text.AbstractReusableInformationControlCreator;
-import org.eclipse.jface.text.DefaultInformationControl;
-import org.eclipse.jface.text.IInformationControl;
 import org.eclipse.jface.viewers.ColumnWeightData;
 import org.eclipse.jface.viewers.IOpenListener;
 import org.eclipse.jface.viewers.ISelection;
@@ -75,6 +71,7 @@ import org.eclipse.jgit.util.FileUtils;
 import org.eclipse.jgit.util.RawParseUtils;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.SWTException;
 import org.eclipse.swt.dnd.Clipboard;
 import org.eclipse.swt.dnd.DND;
 import org.eclipse.swt.dnd.DragSourceAdapter;
@@ -88,17 +85,14 @@ import org.eclipse.swt.events.FocusEvent;
 import org.eclipse.swt.events.FocusListener;
 import org.eclipse.swt.events.MenuDetectEvent;
 import org.eclipse.swt.events.MenuDetectListener;
-import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.FontData;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Point;
-import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Shell;
 import org.eclipse.swt.widgets.Table;
 import org.eclipse.swt.widgets.TableColumn;
 import org.eclipse.swt.widgets.TableItem;
@@ -165,6 +159,8 @@ class CommitGraphTable {
 
 	private boolean trace = GitTraceLocation.HISTORYVIEW.isActive();
 
+	private boolean enableAntialias = true;
+
 	CommitGraphTable(Composite parent, final TableLoader loader,
 			final ResourceManager resources) {
 		nFont = UIUtils.getFont(UIPreferences.THEME_CommitGraphNormalFont);
@@ -235,7 +231,8 @@ class CommitGraphTable {
 			}
 		});
 
-		final RefHoverInformationControlManager hoverManager = new RefHoverInformationControlManager();
+		final CommitGraphTableHoverManager hoverManager = new CommitGraphTableHoverManager(
+				table, renderer);
 		hoverManager.install(table.getTable());
 
 		table.getTable().addDisposeListener(new DisposeListener() {
@@ -281,10 +278,8 @@ class CommitGraphTable {
 				if (input == null || !input.isSingleFile())
 					return;
 
-				ICommandService srv = (ICommandService) site
-						.getService(ICommandService.class);
-				IHandlerService hsrv = (IHandlerService) site
-						.getService(IHandlerService.class);
+				ICommandService srv = CommonUtils.getService(site, ICommandService.class);
+				IHandlerService hsrv = CommonUtils.getService(site, IHandlerService.class);
 				Command cmd = srv.getCommand(HistoryViewCommands.SHOWVERSIONS);
 				Parameterization[] parms;
 				if (Activator.getDefault().getPreferenceStore().getBoolean(
@@ -483,6 +478,15 @@ class CommitGraphTable {
 	}
 
 	void doPaint(final Event event) {
+		// enable antialiasing early to avoid different font extent in
+		// PlotRenderer
+		if (this.enableAntialias)
+			try {
+				event.gc.setAntialias(SWT.ON);
+			} catch (SWTException e) {
+				this.enableAntialias = false;
+			}
+
 		final RevCommit c = (RevCommit) ((TableItem) event.item).getData();
 		if (c instanceof SWTCommit) {
 			final SWTLane lane = ((SWTCommit) c).getLane();
@@ -550,90 +554,6 @@ class CommitGraphTable {
 		return action;
 	}
 
-	private static final class RefHoverInformationControlCreator extends
-			AbstractReusableInformationControlCreator {
-		@Override
-		protected IInformationControl doCreateInformationControl(Shell parent) {
-			return new DefaultInformationControl(parent);
-		}
-	}
-
-	private final class RefHoverInformationControlManager extends
-			AbstractHoverInformationControlManager {
-
-		protected RefHoverInformationControlManager() {
-			super(new RefHoverInformationControlCreator());
-		}
-
-		@Override
-		protected void computeInformation() {
-			MouseEvent e = getHoverEvent();
-
-			TableItem item = table.getTable().getItem(new Point(e.x, e.y));
-			if (item != null) {
-				SWTCommit commit = (SWTCommit) item.getData();
-				if (commit != null && commit.getRefCount() > 0) {
-					Rectangle itemBounds = item.getBounds();
-					int firstColumnWidth = table.getTable().getColumn(0).getWidth();
-					int relativeX = e.x - firstColumnWidth - itemBounds.x;
-					for (int i = 0; i < commit.getRefCount(); i++) {
-						Ref ref = commit.getRef(i);
-						Point textSpan = renderer.getRefHSpan(ref);
-						if ((textSpan != null)
-								&& (relativeX >= textSpan.x && relativeX <= textSpan.y)) {
-
-							String hoverText = getHoverText(ref, i, commit);
-							int width = textSpan.y - textSpan.x;
-							Rectangle rectangle = new Rectangle(
-									firstColumnWidth + itemBounds.x
-											+ textSpan.x, itemBounds.y, width,
-									itemBounds.height);
-							setInformation(hoverText, rectangle);
-							return;
-						}
-					}
-				}
-			}
-
-			// computeInformation must setInformation in all cases
-			setInformation(null, null);
-		}
-
-		private String getHoverText(Ref ref, int refIndex, SWTCommit commit) {
-			if (ref.getName().startsWith(Constants.R_TAGS)
-					&& renderer.isShownAsEllipsis(ref)) {
-				StringBuilder sb = new StringBuilder(UIText.CommitGraphTable_HoverAdditionalTags);
-				for (int i = refIndex; i < commit.getRefCount(); i++) {
-					Ref tag = commit.getRef(i);
-					String name = tag.getName();
-					if (name.startsWith(Constants.R_TAGS)) {
-						sb.append('\n');
-						sb.append(name.substring(Constants.R_TAGS.length()));
-					}
-				}
-				return sb.toString();
-			} else {
-				return getHoverTextForSingleRef(ref);
-			}
-		}
-
-		private String getHoverTextForSingleRef(Ref r) {
-			StringBuilder sb = new StringBuilder();
-			String name = r.getName();
-			sb.append(name);
-			if (r.isSymbolic()) {
-				sb.append(": "); //$NON-NLS-1$
-				sb.append(r.getLeaf().getName());
-			}
-			String description = GitLabelProvider.getRefDescription(r);
-			if (description != null) {
-				sb.append("\n"); //$NON-NLS-1$
-				sb.append(description);
-			}
-			return sb.toString();
-		}
-	}
-
 	private final class CommitDragSourceListener extends DragSourceAdapter {
 		@Override
 		public void dragStart(DragSourceEvent event) {
@@ -760,12 +680,16 @@ class CommitGraphTable {
 		public void menuDetected(MenuDetectEvent e) {
 			popupMgr.removeAll();
 
+			final HistoryPageInput lastInput = this.input;
+			if (lastInput == null)
+				return;
+
 			int selectionSize = ((IStructuredSelection) selectionProvider
 					.getSelection()).size();
 
-			if (input.isSingleFile()) {
+			if (lastInput.isSingleFile()) {
 				if (selectionSize == 1)
-					if (input.getSingleFile() instanceof IResource)
+					if (lastInput.getSingleFile() instanceof IResource)
 						popupMgr
 								.add(getCommandContributionItem(
 										HistoryViewCommands.COMPARE_WITH_TREE,
@@ -797,8 +721,8 @@ class CommitGraphTable {
 
 			if (selectionSize == 1) {
 				popupMgr.add(new Separator());
-				if (!input.getRepository().isBare()) {
-					if (hasMultipleRefNodes()) {
+				if (!lastInput.getRepository().isBare()) {
+					if (hasMultipleRefNodes(lastInput)) {
 						popupMgr.add(getCommandContributionItem(
 								HistoryViewCommands.CHECKOUT,
 								UIText.GitHistoryPage_CheckoutMenuLabel2));
@@ -825,6 +749,9 @@ class CommitGraphTable {
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.CREATE_TAG,
 						UIText.GitHistoryPage_CreateTagMenuLabel));
+				popupMgr.add(getCommandContributionItem(
+						HistoryViewCommands.DELETE_TAG,
+						UIText.CommitGraphTable_DeleteTagAction));
 				popupMgr.add(new Separator());
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.CREATE_PATCH,
@@ -832,9 +759,6 @@ class CommitGraphTable {
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.CHERRYPICK,
 						UIText.GitHistoryPage_cherryPickMenuItem));
-				popupMgr.add(getCommandContributionItem(
-						HistoryViewCommands.REVERT,
-						UIText.GitHistoryPage_revertMenuItem));
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.MERGE,
 						UIText.GitHistoryPage_mergeMenuItem));
@@ -852,12 +776,18 @@ class CommitGraphTable {
 				popupMgr.add(getCommandContributionItem(
 						HistoryViewCommands.COMPARE_VERSIONS,
 						UIText.GitHistoryPage_CompareWithEachOtherMenuLabel));
-				if (!input.isSingleFile())
+				if (!lastInput.isSingleFile())
 					popupMgr
 							.add(getCommandContributionItem(
 									HistoryViewCommands.COMPARE_VERSIONS_IN_TREE,
 									UIText.CommitGraphTable_CompareWithEachOtherInTreeMenuLabel));
 			}
+
+			popupMgr.add(new Separator());
+
+			popupMgr.add(getCommandContributionItem(HistoryViewCommands.REVERT,
+					UIText.GitHistoryPage_revertMenuItem));
+
 			popupMgr.add(new Separator());
 
 			MenuManager quickDiffManager = new MenuManager(
@@ -910,9 +840,9 @@ class CommitGraphTable {
 			popupMgr.add(new Separator());
 		}
 
-		private boolean hasMultipleRefNodes() {
+		private boolean hasMultipleRefNodes(HistoryPageInput lastInput) {
 			try {
-				Map<String, Ref> branches = input.getRepository()
+				Map<String, Ref> branches = lastInput.getRepository()
 						.getRefDatabase().getRefs(Constants.R_HEADS);
 				int count = 0;
 				for (Ref branch : branches.values()) {
