@@ -13,14 +13,9 @@
 
 package org.eclipse.egit.ui.internal.history;
 
-import java.io.BufferedOutputStream;
-import java.io.ByteArrayOutputStream;
 import java.io.IOException;
-import java.io.OutputStream;
-import java.io.UnsupportedEncodingException;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
@@ -31,14 +26,11 @@ import java.util.regex.Pattern;
 
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.OperationCanceledException;
-import org.eclipse.egit.core.internal.CompareCoreUtils;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.history.CommitMessageViewer.ObjectLink;
 import org.eclipse.egit.ui.internal.trace.GitTraceLocation;
-import org.eclipse.jgit.diff.DiffFormatter;
-import org.eclipse.jgit.diff.RawText;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.lib.Constants;
@@ -51,7 +43,6 @@ import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevObject;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.revwalk.RevWalkUtils;
-import org.eclipse.jgit.util.io.SafeBufferedOutputStream;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyleRange;
@@ -76,34 +67,24 @@ public class CommitInfoBuilder {
 
 	private final boolean fill;
 
-	// set by selecting files in the file list
-	private final List<FileDiff> currentDiffs;
-
 	private Color linkColor;
 
 	private Color darkGrey;
 
-	private Color hunkheaderColor;
-
-	private Color linesAddedColor;
-
-	private Color linesRemovedColor;
-
 	private final Collection<Ref> allRefs;
 
 	/**
 	 * @param db the repository
 	 * @param commit the commit the info should be shown for
-	 * @param currentDiffs list of current diffs
 	 * @param fill whether to fill the available space
 	 * @param allRefs all Ref's to examine regarding marge bases
 	 */
-	public CommitInfoBuilder(Repository db, PlotCommit commit, List<FileDiff> currentDiffs, boolean fill, Collection<Ref> allRefs) {
+	public CommitInfoBuilder(Repository db, PlotCommit commit, boolean fill,
+			Collection<Ref> allRefs) {
 		this.db = db;
 		this.commit = commit;
 		this.fill = fill;
 		this.allRefs = allRefs;
-		this.currentDiffs = new ArrayList<FileDiff>(currentDiffs);
 	}
 
 	/**
@@ -111,16 +92,10 @@ public class CommitInfoBuilder {
 	 *
 	 * @param linkColor
 	 * @param darkGrey
-	 * @param hunkheaderColor
-	 * @param linesAddedColor
-	 * @param linesRemovedColor
 	 */
-	public void setColors(Color linkColor, Color darkGrey, Color hunkheaderColor, Color linesAddedColor, Color linesRemovedColor) {
+	public void setColors(Color linkColor, Color darkGrey) {
 		this.linkColor = linkColor;
 		this.darkGrey = darkGrey;
-		this.hunkheaderColor = hunkheaderColor;
-		this.linesAddedColor = linesAddedColor;
-		this.linesRemovedColor = linesRemovedColor;
 	}
 
 	/**
@@ -282,9 +257,6 @@ public class CommitInfoBuilder {
 					- matcher.start(), null, null, SWT.ITALIC));
 		}
 
-		if (!currentDiffs.isEmpty())
-			buildDiffs(d, styles, monitor, trace);
-
 		if (trace)
 			GitTraceLocation.getTrace().traceExit(
 					GitTraceLocation.HISTORYVIEW.getLocation());
@@ -368,90 +340,6 @@ public class CommitInfoBuilder {
 		}
 	}
 
-
-	private void buildDiffs(final StringBuilder d,
-			final List<StyleRange> styles, IProgressMonitor monitor,
-			boolean trace) throws OperationCanceledException,
-			IOException {
-
-		// the encoding for the currently processed file
-		final String[] currentEncoding = new String[1];
-
-		if (trace)
-			GitTraceLocation.getTrace().traceEntry(
-					GitTraceLocation.HISTORYVIEW.getLocation());
-		if (commit.getParentCount() > 1) {
-			d.append(UIText.CommitMessageViewer_CanNotRenderDiffMessage);
-			return;
-		}
-
-		try {
-			monitor.beginTask(UIText.CommitMessageViewer_BuildDiffListTaskName,
-					currentDiffs.size());
-			BufferedOutputStream bos = new SafeBufferedOutputStream(
-					new ByteArrayOutputStream() {
-						@Override
-						public synchronized void write(byte[] b, int off,
-								int len) {
-							super.write(b, off, len);
-							try {
-								if (currentEncoding[0] == null)
-									d.append(toString("UTF-8")); //$NON-NLS-1$
-								else
-									d.append(toString(currentEncoding[0]));
-							} catch (UnsupportedEncodingException e) {
-								d.append(toString());
-							}
-							reset();
-						}
-
-					});
-			final DiffFormatter diffFmt = new MessageViewerFormatter(bos,
-					styles, d, hunkheaderColor, linesAddedColor, linesRemovedColor);
-
-			for (FileDiff currentDiff : currentDiffs) {
-				if (monitor.isCanceled())
-					throw new OperationCanceledException();
-				if (currentDiff.getBlobs().length == 2) {
-					String path = currentDiff.getNewPath();
-					monitor.setTaskName(NLS.bind(
-							UIText.CommitMessageViewer_BuildDiffTaskName, path));
-					currentEncoding[0] = CompareCoreUtils.getResourceEncoding(db,
-							path);
-					d.append(LF);
-					int start = d.length();
-					String pathLine = formatPathLine(path);
-					int len = pathLine.length();
-					d.append(pathLine).append(LF);
-					styles.add(new StyleRange(start, len, darkGrey, null));
-					currentDiff.outputDiff(d, db, diffFmt, true);
-					diffFmt.flush();
-				}
-				monitor.worked(1);
-			}
-
-		} finally {
-			monitor.done();
-			if (trace)
-				GitTraceLocation.getTrace().traceExit(
-						GitTraceLocation.HISTORYVIEW.getLocation());
-		}
-	}
-
-	private String formatPathLine(String path) {
-		int n = 80 - path.length() - 2;
-		if (n < 0)
-			return path;
-		final StringBuilder d = new StringBuilder();
-		int i = 0;
-		for (; i < n / 2; i++)
-			d.append("-"); //$NON-NLS-1$
-		d.append(SPACE).append(path).append(SPACE);
-		for (; i < n - 1; i++)
-			d.append("-"); //$NON-NLS-1$
-		return d.toString();
-	}
-
 	private String getTagsString() {
 		StringBuilder sb = new StringBuilder();
 		Map<String, Ref> tagsMap = db.getTags();
@@ -468,60 +356,6 @@ public class CommitInfoBuilder {
 		return sb.toString();
 	}
 
-	private static final class MessageViewerFormatter extends DiffFormatter {
-		private final List<StyleRange> styles;
-
-		private final StringBuilder d;
-
-		private final Color hunkheaderColor;
-		private final Color linesAddedColor;
-		private final Color linesRemovedColor;
-
-		private MessageViewerFormatter(OutputStream out,
-				List<StyleRange> styles, StringBuilder d, Color hunkheaderColor, Color linesAddedColor, Color linesRemovedColor) {
-			super(out);
-			this.styles = styles;
-			this.hunkheaderColor = hunkheaderColor;
-			this.linesAddedColor = linesAddedColor;
-			this.linesRemovedColor = linesRemovedColor;
-			this.d = d;
-		}
-
-		@Override
-		protected void writeHunkHeader(int aCur, int aEnd, int bCur, int bEnd)
-				throws IOException {
-			flush();
-			int start = d.length();
-			super.writeHunkHeader(aCur, aEnd, bCur, bEnd);
-			flush();
-			int end = d.length();
-			styles.add(new StyleRange(start, end - start, hunkheaderColor,
-					null));
-		}
-
-		@Override
-		protected void writeAddedLine(RawText b, int bCur) throws IOException {
-			flush();
-			int start = d.length();
-			super.writeAddedLine(b, bCur);
-			flush();
-			int end = d.length();
-			styles.add(new StyleRange(start, end - start,
-					linesAddedColor, null));
-		}
-
-		@Override
-		protected void writeRemovedLine(RawText b, int bCur) throws IOException {
-			flush();
-			int start = d.length();
-			super.writeRemovedLine(b, bCur);
-			flush();
-			int end = d.length();
-			styles.add(new StyleRange(start, end - start,
-					linesRemovedColor, null));
-		}
-	}
-
 	/**
 	 * Finds next door tagged revision. Searches forwards (in descendants) or
 	 * backwards (in ancestors)
