@@ -74,6 +74,8 @@ public class FindToolbar extends Composite {
 
 	private static final int PREFS_FINDIN_COMMITTER = 4;
 
+	private static final int PREFS_FINDIN_REFERENCE = 5;
+
 	private Color errorBackgroundColor;
 
 	/**
@@ -117,6 +119,8 @@ public class FindToolbar extends Composite {
 
 	private MenuItem committerItem;
 
+	private MenuItem referenceItem;
+
 	private Image nextIcon;
 
 	private Image previousIcon;
@@ -131,6 +135,8 @@ public class FindToolbar extends Composite {
 
 	private Image committerIcon;
 
+	private Image branchesIcon;
+
 	/**
 	 * Creates the toolbar.
 	 *
@@ -151,6 +157,7 @@ public class FindToolbar extends Composite {
 		commentsIcon = UIIcons.ELCL16_COMMENTS.createImage();
 		authorIcon = UIIcons.ELCL16_AUTHOR.createImage();
 		committerIcon = UIIcons.ELCL16_COMMITTER.createImage();
+		branchesIcon = UIIcons.BRANCHES.createImage();
 
 		GridLayout findLayout = new GridLayout();
 		findLayout.marginHeight = 2;
@@ -201,6 +208,9 @@ public class FindToolbar extends Composite {
 		committerItem = createFindInMenuItem();
 		committerItem.setText(UIText.HistoryPage_findbar_committer);
 		committerItem.setImage(committerIcon);
+		referenceItem = createFindInMenuItem();
+		referenceItem.setText(UIText.HistoryPage_findbar_reference);
+		referenceItem.setImage(branchesIcon);
 
 		prefsDropDown.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event event) {
@@ -222,6 +232,8 @@ public class FindToolbar extends Composite {
 					else if (commitIdItem.getSelection())
 						selectFindInItem(committerItem);
 					else if (committerItem.getSelection())
+						selectFindInItem(referenceItem);
+					else if (referenceItem.getSelection())
 						selectFindInItem(allItem);
 				}
 			}
@@ -341,6 +353,8 @@ public class FindToolbar extends Composite {
 			selectFindInItem(commitIdItem);
 		else if (selectedPrefsItem == PREFS_FINDIN_COMMITTER)
 			selectFindInItem(committerItem);
+		else if (selectedPrefsItem == PREFS_FINDIN_REFERENCE)
+			selectFindInItem(referenceItem);
 
 		registerDisposal();
 	}
@@ -358,6 +372,7 @@ public class FindToolbar extends Composite {
 				commentsIcon.dispose();
 				authorIcon.dispose();
 				committerIcon.dispose();
+				branchesIcon.dispose();
 			}
 		});
 	}
@@ -387,6 +402,9 @@ public class FindToolbar extends Composite {
 					UIText.HistoryPage_findbar_changeto_committer);
 		else if (menuItem == committerItem)
 			selectFindInItem(menuItem, PREFS_FINDIN_COMMITTER, committerIcon,
+					UIText.HistoryPage_findbar_changeto_reference);
+		else if (menuItem == referenceItem)
+			selectFindInItem(menuItem, PREFS_FINDIN_REFERENCE, branchesIcon,
 					UIText.HistoryPage_findbar_changeto_all);
 	}
 
@@ -411,6 +429,7 @@ public class FindToolbar extends Composite {
 		commentsItem.setSelection(false);
 		authorItem.setSelection(false);
 		committerItem.setSelection(false);
+		referenceItem.setSelection(false);
 		item.setSelection(true);
 		clear();
 	}
@@ -426,11 +445,13 @@ public class FindToolbar extends Composite {
 			finder.findInComments = true;
 			finder.findInAuthor = true;
 			finder.findInCommitter = true;
+			finder.findInReference = true;
 		} else {
 			finder.findInCommitId = commitIdItem.getSelection();
 			finder.findInComments = commentsItem.getSelection();
 			finder.findInAuthor = authorItem.getSelection();
 			finder.findInCommitter = committerItem.getSelection();
+			finder.findInReference = referenceItem.getSelection();
 		}
 		return finder;
 	}
