@@ -9,18 +9,14 @@
 package org.eclipse.egit.ui.internal.history.command;
 
 import java.io.File;
-import java.io.IOException;
 import java.util.Iterator;
 
 import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IResource;
-import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.Path;
-import org.eclipse.egit.ui.Activator;
+import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.internal.CompareUtils;
-import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.history.GitHistoryPage;
 import org.eclipse.egit.ui.internal.merge.GitCompareEditorInput;
 import org.eclipse.jface.viewers.IStructuredSelection;
@@ -46,26 +42,21 @@ public class CompareVersionsHandler extends AbstractHistoryCommandHandler {
 			IWorkbenchPage workBenchPage = HandlerUtil
 					.getActiveWorkbenchWindowChecked(event).getActivePage();
 			if (input instanceof IFile) {
-				IResource[] resources = new IResource[] { (IFile) input, };
-				try {
-					CompareUtils.compare(resources, repo, commit1.getName(),
-							commit2.getName(), false, workBenchPage);
-				} catch (IOException e) {
-					Activator.handleError(
-							UIText.CompareWithRefAction_errorOnSynchronize, e,
-							true);
-				}
+				IFile resource = (IFile) input;
+				final RepositoryMapping map = RepositoryMapping
+						.getMapping(resource);
+				final String gitPath = map.getRepoRelativePath(resource);
+				final String commit1Path = getRenamedPath(gitPath, commit1);
+				final String commit2Path = getRenamedPath(gitPath, commit2);
+				CompareUtils.openInCompare(commit1, commit2, commit1Path,
+						commit2Path, map.getRepository(), workBenchPage);
 			} else if (input instanceof File) {
 				File fileInput = (File) input;
-				IPath location = new Path(fileInput.getAbsolutePath());
-				try {
-					CompareUtils.compare(location, repo, commit1.getName(),
-							commit2.getName(), false, workBenchPage);
-				} catch (IOException e) {
-					Activator.handleError(
-							UIText.CompareWithRefAction_errorOnSynchronize, e,
-							true);
-				}
+				final String gitPath = getRepoRelativePath(repo, fileInput);
+				final String commit1Path = getRenamedPath(gitPath, commit1);
+				final String commit2Path = getRenamedPath(gitPath, commit2);
+				CompareUtils.openInCompare(commit1, commit2, commit1Path,
+						commit2Path, repo, workBenchPage);
 			} else if (input instanceof IResource) {
 				GitCompareEditorInput compareInput = new GitCompareEditorInput(
 						commit1.name(), commit2.name(), (IResource) input);
