@@ -11,8 +11,6 @@ package org.eclipse.egit.ui.internal.history.command;
 import java.io.File;
 import java.util.Iterator;
 
-import org.eclipse.compare.CompareEditorInput;
-import org.eclipse.compare.ITypedElement;
 import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.resources.IFile;
@@ -20,7 +18,6 @@ import org.eclipse.core.resources.IResource;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.internal.CompareUtils;
-import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput;
 import org.eclipse.egit.ui.internal.dialogs.CompareTreeView;
 import org.eclipse.egit.ui.internal.history.GitHistoryPage;
 import org.eclipse.egit.ui.internal.history.HistoryPageInput;
@@ -56,28 +53,20 @@ public class CompareVersionsInTreeHandler extends
 				final RepositoryMapping map = RepositoryMapping
 						.getMapping(resource);
 				final String gitPath = map.getRepoRelativePath(resource);
+				final String commit1Path = getRenamedPath(gitPath, commit1);
+				final String commit2Path = getRenamedPath(gitPath, commit2);
 
-				final ITypedElement base = CompareUtils
-						.getFileRevisionTypedElement(gitPath, commit1, map
-								.getRepository());
-				final ITypedElement next = CompareUtils
-						.getFileRevisionTypedElement(gitPath, commit2, map
-								.getRepository());
-				CompareEditorInput in = new GitCompareFileRevisionEditorInput(
-						base, next, null);
-				CompareUtils.openInCompare(workBenchPage, in);
+				CompareUtils.openInCompare(commit1, commit2, commit1Path,
+						commit2Path, map.getRepository(), workBenchPage);
 			} else if (input instanceof File) {
 				File fileInput = (File) input;
 				Repository repo = getRepository(event);
 				final String gitPath = getRepoRelativePath(repo, fileInput);
+				final String commit1Path = getRenamedPath(gitPath, commit1);
+				final String commit2Path = getRenamedPath(gitPath, commit2);
 
-				final ITypedElement base = CompareUtils
-						.getFileRevisionTypedElement(gitPath, commit1, repo);
-				final ITypedElement next = CompareUtils
-						.getFileRevisionTypedElement(gitPath, commit2, repo);
-				CompareEditorInput in = new GitCompareFileRevisionEditorInput(
-						base, next, null);
-				CompareUtils.openInCompare(workBenchPage, in);
+				CompareUtils.openInCompare(commit1, commit2, commit1Path,
+						commit2Path, repo, workBenchPage);
 			} else if (input instanceof IResource) {
 				CompareTreeView view;
 				try {
