@@ -50,6 +50,9 @@ public class HistoryViewCommands {
 	/** "Create Tag" */
 	public static final String CREATE_TAG = "org.eclipse.egit.ui.history.CreateTag"; //$NON-NLS-1$
 
+	/** "Delete Tag" */
+	public static final String DELETE_TAG = "org.eclipse.egit.ui.history.DeleteTag"; //$NON-NLS-1$
+
 	/** "Push Commit" */
 	public static final String PUSH_COMMIT = "org.eclipse.egit.ui.history.PushCommit"; //$NON-NLS-1$
 
