@@ -54,17 +54,18 @@ public class OpenInTextEditorHandler extends AbstractHistoryCommandHandler {
 			Iterator<?> it = selection.iterator();
 			while (it.hasNext()) {
 				RevCommit commit = (RevCommit) it.next();
+				String commitPath = getRenamedPath(gitPath, commit);
 				IFileRevision rev = null;
 				try {
-					rev = CompareUtils.getFileRevision(gitPath, commit, map
-							.getRepository(), null);
+					rev = CompareUtils.getFileRevision(commitPath, commit,
+							map.getRepository(), null);
 				} catch (IOException e) {
 					Activator.logError(NLS.bind(
-							UIText.GitHistoryPage_errorLookingUpPath, gitPath,
-							commit.getId()), e);
+							UIText.GitHistoryPage_errorLookingUpPath,
+							commitPath, commit.getId()), e);
 					errorOccurred = true;
 				}
-				if (rev != null) {
+				if (rev != null)
 					try {
 						EgitUiEditorUtils.openTextEditor(getPart(event)
 								.getSite().getPage(), rev, null);
@@ -72,9 +73,8 @@ public class OpenInTextEditorHandler extends AbstractHistoryCommandHandler {
 						Activator.logError(e.getMessage(), e);
 						errorOccurred = true;
 					}
-				} else {
+				else
 					ids.add(commit.getId());
-				}
 			}
 		}
 		if (input instanceof File) {
@@ -84,17 +84,18 @@ public class OpenInTextEditorHandler extends AbstractHistoryCommandHandler {
 			Iterator<?> it = selection.iterator();
 			while (it.hasNext()) {
 				RevCommit commit = (RevCommit) it.next();
+				String commitPath = getRenamedPath(gitPath, commit);
 				IFileRevision rev = null;
 				try {
-					rev = CompareUtils.getFileRevision(gitPath, commit, repo,
-							null);
+					rev = CompareUtils.getFileRevision(commitPath, commit,
+							repo, null);
 				} catch (IOException e) {
 					Activator.logError(NLS.bind(
-							UIText.GitHistoryPage_errorLookingUpPath, gitPath,
-							commit.getId()), e);
+							UIText.GitHistoryPage_errorLookingUpPath,
+							commitPath, commit.getId()), e);
 					errorOccurred = true;
 				}
-				if (rev != null) {
+				if (rev != null)
 					try {
 						EgitUiEditorUtils.openTextEditor(getPart(event)
 								.getSite().getPage(), rev, null);
@@ -102,9 +103,8 @@ public class OpenInTextEditorHandler extends AbstractHistoryCommandHandler {
 						Activator.logError(e.getMessage(), e);
 						errorOccurred = true;
 					}
-				} else {
+				else
 					ids.add(commit.getId());
-				}
 			}
 		}
 		if (errorOccurred)
