@@ -26,6 +26,7 @@ import org.eclipse.compare.structuremergeviewer.IDiffContainer;
 import org.eclipse.compare.structuremergeviewer.IDiffElement;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
@@ -36,13 +37,13 @@ import org.eclipse.egit.core.internal.storage.WorkingTreeFileRevision;
 import org.eclipse.egit.core.internal.util.ResourceUtil;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.internal.CompareUtils;
-import org.eclipse.egit.ui.internal.EditableRevision;
-import org.eclipse.egit.ui.internal.FileRevisionTypedElement;
-import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput.EmptyTypedElement;
-import org.eclipse.egit.ui.internal.LocalFileRevision;
-import org.eclipse.egit.ui.internal.LocationEditableRevision;
-import org.eclipse.egit.ui.internal.ResourceEditableRevision;
 import org.eclipse.egit.ui.internal.UIText;
+import org.eclipse.egit.ui.internal.revision.EditableRevision;
+import org.eclipse.egit.ui.internal.revision.FileRevisionTypedElement;
+import org.eclipse.egit.ui.internal.revision.GitCompareFileRevisionEditorInput.EmptyTypedElement;
+import org.eclipse.egit.ui.internal.revision.LocalFileRevision;
+import org.eclipse.egit.ui.internal.revision.LocationEditableRevision;
+import org.eclipse.egit.ui.internal.revision.ResourceEditableRevision;
 import org.eclipse.jface.operation.IRunnableContext;
 import org.eclipse.jgit.api.RebaseCommand;
 import org.eclipse.jgit.dircache.DirCacheEntry;
@@ -103,6 +104,22 @@ public class GitMergeEditorInput extends CompareEditorInput {
 		config.setLeftEditable(true);
 	}
 
+	@Override
+	public Object getAdapter(Class adapter) {
+		if (adapter == IFile.class || adapter == IResource.class) {
+			Object selectedEdition = getSelectedEdition();
+			if (selectedEdition instanceof DiffNode) {
+				DiffNode diffNode = (DiffNode) selectedEdition;
+				ITypedElement element = diffNode.getLeft();
+				if (element instanceof ResourceEditableRevision) {
+					ResourceEditableRevision resourceRevision = (ResourceEditableRevision) element;
+					return resourceRevision.getFile();
+				}
+			}
+		}
+		return super.getAdapter(adapter);
+	}
+
 	@Override
 	protected Object prepareInput(IProgressMonitor monitor)
 			throws InvocationTargetException, InterruptedException {
