@@ -571,21 +571,30 @@ public class SimpleConfigurePushDialog extends TitleAreaDialog {
 	public void buttonPressed(int buttonId) {
 		if (buttonId == DRY_RUN) {
 			try {
-				new ProgressMonitorDialog(getShell()).run(false, true,
+				new ProgressMonitorDialog(getShell()).run(true, true,
 						new IRunnableWithProgress() {
 							public void run(IProgressMonitor monitor)
 									throws InvocationTargetException,
 									InterruptedException {
-								PushOperationUI op = new PushOperationUI(
+								final PushOperationUI op = new PushOperationUI(
 										repository, config, true);
 								try {
-									PushOperationResult result = op
+									final PushOperationResult result = op
 											.execute(monitor);
-									PushResultDialog dlg = new PushResultDialog(
-											getShell(), repository, result, op
+									getShell().getDisplay().asyncExec(
+											new Runnable() {
+
+												public void run() {
+													PushResultDialog dlg = new PushResultDialog(
+															getShell(),
+															repository,
+															result,
+															op
 													.getDestinationString());
-									dlg.showConfigureButton(false);
-									dlg.open();
+													dlg.showConfigureButton(false);
+													dlg.open();
+												}
+											});
 								} catch (CoreException e) {
 									Activator.handleError(e.getMessage(), e,
 											true);
@@ -618,7 +627,7 @@ public class SimpleConfigurePushDialog extends TitleAreaDialog {
 			}
 			if (buttonId == OK)
 				try {
-					new ProgressMonitorDialog(getShell()).run(false, true,
+					new ProgressMonitorDialog(getShell()).run(true, true,
 							new IRunnableWithProgress() {
 								public void run(IProgressMonitor monitor)
 										throws InvocationTargetException,
