@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2013 SAP AG.
+ * Copyright (c) 2013, 2014 SAP AG and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,10 +10,19 @@
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.rebase;
 
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.List;
+
 import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.IAdaptable;
+import org.eclipse.core.runtime.preferences.IEclipsePreferences.IPreferenceChangeListener;
+import org.eclipse.core.runtime.preferences.IEclipsePreferences.PreferenceChangeEvent;
+import org.eclipse.core.runtime.preferences.InstanceScope;
 import org.eclipse.egit.core.AdapterUtils;
+import org.eclipse.egit.core.RepositoryUtil;
 import org.eclipse.egit.core.internal.rebase.RebaseInteractivePlan;
 import org.eclipse.egit.core.internal.rebase.RebaseInteractivePlan.ElementAction;
 import org.eclipse.egit.core.internal.rebase.RebaseInteractivePlan.ElementType;
@@ -22,31 +31,48 @@ import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIUtils;
+import org.eclipse.egit.ui.internal.CommonUtils;
 import org.eclipse.egit.ui.internal.UIIcons;
 import org.eclipse.egit.ui.internal.UIText;
+import org.eclipse.egit.ui.internal.actions.BooleanPrefAction;
 import org.eclipse.egit.ui.internal.commands.shared.AbortRebaseCommand;
 import org.eclipse.egit.ui.internal.commands.shared.AbstractRebaseCommandHandler;
 import org.eclipse.egit.ui.internal.commands.shared.ContinueRebaseCommand;
 import org.eclipse.egit.ui.internal.commands.shared.ProcessStepsRebaseCommand;
 import org.eclipse.egit.ui.internal.commands.shared.SkipRebaseCommand;
+import org.eclipse.egit.ui.internal.commit.CommitEditor;
+import org.eclipse.egit.ui.internal.commit.RepositoryCommit;
 import org.eclipse.egit.ui.internal.repository.RepositoriesView;
 import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNode;
+import org.eclipse.jface.action.Action;
+import org.eclipse.jface.action.IMenuListener;
+import org.eclipse.jface.action.IMenuManager;
+import org.eclipse.jface.action.IToolBarManager;
+import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.layout.GridDataFactory;
 import org.eclipse.jface.layout.GridLayoutFactory;
+import org.eclipse.jface.preference.IPersistentPreferenceStore;
 import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.jface.resource.LocalResourceManager;
 import org.eclipse.jface.util.LocalSelectionTransfer;
 import org.eclipse.jface.viewers.ColumnLabelProvider;
 import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
+import org.eclipse.jface.viewers.DoubleClickEvent;
+import org.eclipse.jface.viewers.IDoubleClickListener;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
+import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jface.viewers.TreeViewer;
 import org.eclipse.jface.viewers.TreeViewerColumn;
 import org.eclipse.jface.window.ToolTip;
+import org.eclipse.jgit.lib.AbbreviatedObjectId;
+import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryState;
+import org.eclipse.jgit.revwalk.RevCommit;
+import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.util.GitDateFormatter;
 import org.eclipse.jgit.util.GitDateFormatter.Format;
 import org.eclipse.swt.SWT;
@@ -61,9 +87,13 @@ import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.RowLayout;
 import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.Menu;
 import org.eclipse.swt.widgets.ToolBar;
 import org.eclipse.swt.widgets.ToolItem;
 import org.eclipse.swt.widgets.Tree;
+import org.eclipse.swt.widgets.TreeItem;
+import org.eclipse.ui.IActionBars;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorPart;
 import org.eclipse.ui.IFileEditorInput;
@@ -123,6 +153,12 @@ public class RebaseInteractiveView extends ViewPart implements
 	/** these columns are dynamically resized to fit their contents */
 	private TreeViewerColumn[] dynamicColumns;
 
+	private List<PlanContextMenuAction> contextMenuItems;
+
+	private RebasePlanIndexer planIndexer;
+
+	private IPreferenceChangeListener prefListener;
+
 	/**
 	 * View for handling interactive rebase
 	 */
@@ -157,6 +193,9 @@ public class RebaseInteractiveView extends ViewPart implements
 			if (resource != null) {
 				RepositoryMapping mapping = RepositoryMapping
 						.getMapping(resource);
+				if (mapping == null)
+					return;
+
 				repo = mapping.getRepository();
 			}
 		}
@@ -182,10 +221,18 @@ public class RebaseInteractiveView extends ViewPart implements
 	}
 
 	private void removeListeners() {
-		ISelectionService srv = (ISelectionService) getSite().getService(
-				ISelectionService.class);
+		ISelectionService srv = CommonUtils.getService(getSite(), ISelectionService.class);
 		srv.removePostSelectionListener(RepositoriesView.VIEW_ID,
 				selectionChangedListener);
+		if (currentPlan != null)
+			currentPlan.removeRebaseInteractivePlanChangeListener(this);
+
+		if (planIndexer != null)
+			planIndexer.dispose();
+
+		InstanceScope.INSTANCE.getNode(
+				org.eclipse.egit.core.Activator.getPluginId())
+				.removePreferenceChangeListener(prefListener);
 	}
 
 	@Override
@@ -212,6 +259,90 @@ public class RebaseInteractiveView extends ViewPart implements
 
 		setupListeners();
 		createLocalDragandDrop();
+		planTreeViewer.addDoubleClickListener(new IDoubleClickListener() {
+
+			public void doubleClick(DoubleClickEvent event) {
+				PlanElement element = (PlanElement) ((IStructuredSelection) event
+						.getSelection()).getFirstElement();
+				if (element == null)
+					return;
+
+				RepositoryCommit commit = loadCommit(element.getCommit());
+				if (commit != null)
+					CommitEditor.openQuiet(commit);
+
+			}
+
+			private RepositoryCommit loadCommit(
+					AbbreviatedObjectId abbreviatedObjectId) {
+				if (abbreviatedObjectId != null) {
+					RevWalk walk = new RevWalk(
+							RebaseInteractiveView.this.currentRepository);
+					try {
+						Collection<ObjectId> resolved = walk.getObjectReader()
+								.resolve(abbreviatedObjectId);
+						if (resolved.size() == 1) {
+							RevCommit commit = walk.parseCommit(resolved
+									.iterator().next());
+							return new RepositoryCommit(
+									RebaseInteractiveView.this.currentRepository,
+									commit);
+						}
+					} catch (IOException e) {
+						return null;
+					} finally {
+						walk.release();
+					}
+				}
+				return null;
+			}
+		});
+
+		prefListener = new IPreferenceChangeListener() {
+			public void preferenceChange(PreferenceChangeEvent event) {
+				if (!RepositoryUtil.PREFS_DIRECTORIES.equals(event.getKey()))
+					return;
+
+				final Repository repo = currentRepository;
+				if (repo == null)
+					return;
+
+				if (Activator.getDefault().getRepositoryUtil().contains(repo))
+					return;
+
+				// Unselect repository as it has been removed
+				Display.getDefault().asyncExec(new Runnable() {
+					public void run() {
+						currentRepository = null;
+						showRepository(null);
+					}
+				});
+			}
+		};
+
+		InstanceScope.INSTANCE.getNode(
+				org.eclipse.egit.core.Activator.getPluginId())
+				.addPreferenceChangeListener(prefListener);
+
+		IActionBars actionBars = getViewSite().getActionBars();
+		IToolBarManager toolbar = actionBars.getToolBarManager();
+
+		listenOnRepositoryViewSelection = RebaseInteractivePreferences
+				.isReactOnSelection();
+
+		// link with selection
+		Action linkSelectionAction = new BooleanPrefAction(
+				(IPersistentPreferenceStore) Activator.getDefault()
+						.getPreferenceStore(),
+				UIPreferences.REBASE_INTERACTIVE_SYNC_SELECTION,
+				UIText.InteractiveRebaseView_LinkSelection) {
+			@Override
+			public void apply(boolean value) {
+				listenOnRepositoryViewSelection = value;
+			}
+		};
+		linkSelectionAction.setImageDescriptor(UIIcons.ELCL16_SYNCED);
+		toolbar.add(linkSelectionAction);
 	}
 
 	private void createCommandToolBar(Form theForm, FormToolkit toolkit) {
@@ -253,7 +384,7 @@ public class RebaseInteractiveView extends ViewPart implements
 		abortItem.setText(UIText.InteractiveRebaseView_abortItem_text);
 		abortItem.setEnabled(false);
 
-		new ToolItem(toolBar, SWT.SEPARATOR);
+		createSeparator(toolBar);
 
 		refreshItem = new ToolItem(toolBar, SWT.NONE);
 		refreshItem.setImage(UIIcons
@@ -267,6 +398,10 @@ public class RebaseInteractiveView extends ViewPart implements
 		refreshItem.setText(UIText.InteractiveRebaseView_refreshItem_text);
 	}
 
+	private static ToolItem createSeparator(ToolBar toolBar) {
+		return new ToolItem(toolBar, SWT.SEPARATOR);
+	}
+
 	private TreeViewer createPlanTreeViewer(Section rebasePlanSection,
 			FormToolkit toolkit) {
 
@@ -353,10 +488,8 @@ public class RebaseInteractiveView extends ViewPart implements
 			}
 		};
 
-		ISelectionService srv = (ISelectionService) getSite().getService(
-				ISelectionService.class);
-		srv.addPostSelectionListener(RepositoriesView.VIEW_ID,
-				selectionChangedListener);
+		ISelectionService srv = CommonUtils.getService(getSite(), ISelectionService.class);
+		srv.addPostSelectionListener(selectionChangedListener);
 	}
 
 	private class RebaseCommandItemSelectionListener extends SelectionAdapter {
@@ -433,6 +566,7 @@ public class RebaseInteractiveView extends ViewPart implements
 	// show empty space)
 	private void createColumns() {
 		String[] headings = { UIText.RebaseInteractiveView_HeadingStatus,
+				UIText.RebaseInteractiveView_HeadingStep,
 				UIText.RebaseInteractiveView_HeadingAction,
 				UIText.RebaseInteractiveView_HeadingCommitId,
 				UIText.RebaseInteractiveView_HeadingMessage,
@@ -452,8 +586,6 @@ public class RebaseInteractiveView extends ViewPart implements
 				ElementType t = getType(element);
 				if (t != null) {
 					switch (t) {
-					case TODO:
-						return UIIcons.getImage(resources, UIIcons.TODO_STEP);
 					case DONE_CURRENT:
 						return UIIcons
 								.getImage(resources, UIIcons.CURRENT_STEP);
@@ -490,7 +622,44 @@ public class RebaseInteractiveView extends ViewPart implements
 			}
 		});
 
-		TreeViewerColumn actionColumn = createColumn(headings[1], 90);
+		TreeViewerColumn stepColumn = createColumn(headings[1], 55);
+		stepColumn.setLabelProvider(new HighlightingColumnLabelProvider() {
+			@Override
+			public String getText(Object element) {
+				if (element instanceof PlanElement) {
+					PlanElement planLine = (PlanElement) element;
+					return (planIndexer.indexOf(planLine) + 1) + "."; //$NON-NLS-1$
+				}
+				return super.getText(element);
+			}
+		});
+		stepColumn.getColumn().addSelectionListener(new SelectionAdapter() {
+			@Override
+			public void widgetSelected(SelectionEvent e) {
+				Tree tree = planTreeViewer.getTree();
+
+				boolean orderReversed = tree.getSortDirection() == SWT.DOWN;
+
+				RebaseInteractivePreferences.setOrderReversed(!orderReversed);
+
+				int newDirection = (orderReversed ? SWT.UP : SWT.DOWN);
+				tree.setSortDirection(newDirection);
+
+				TreeItem topmostVisibleItem = tree.getTopItem();
+				refreshUI();
+				if (topmostVisibleItem != null)
+					tree.showItem(topmostVisibleItem);
+			}
+		});
+
+		int direction = (RebaseInteractivePreferences.isOrderReversed() ? SWT.DOWN
+				: SWT.UP);
+
+		Tree planTree = planTreeViewer.getTree();
+		planTree.setSortColumn(stepColumn.getColumn());
+		planTree.setSortDirection(direction);
+
+		TreeViewerColumn actionColumn = createColumn(headings[2], 90);
 		actionColumn.setLabelProvider(new HighlightingColumnLabelProvider() {
 
 			@Override
@@ -501,7 +670,12 @@ public class RebaseInteractiveView extends ViewPart implements
 					case EDIT:
 						return UIIcons.getImage(resources, UIIcons.EDITCONFIG);
 					case FIXUP:
-						return UIIcons.getImage(resources, UIIcons.FIXUP);
+						if (RebaseInteractivePreferences.isOrderReversed())
+							return UIIcons.getImage(resources,
+									UIIcons.FIXUP_DOWN);
+						else
+							return UIIcons.getImage(resources,
+									UIIcons.FIXUP_UP);
 					case PICK:
 						return UIIcons.getImage(resources, UIIcons.CHERRY_PICK);
 					case REWORD:
@@ -509,7 +683,12 @@ public class RebaseInteractiveView extends ViewPart implements
 					case SKIP:
 						return UIIcons.getImage(resources, UIIcons.REBASE_SKIP);
 					case SQUASH:
-						return UIIcons.getImage(resources, UIIcons.SQUASH);
+						if (RebaseInteractivePreferences.isOrderReversed())
+							return UIIcons.getImage(resources,
+									UIIcons.SQUASH_DOWN);
+						else
+							return UIIcons.getImage(resources,
+									UIIcons.SQUASH_UP);
 					default:
 						// fall through
 					}
@@ -532,7 +711,7 @@ public class RebaseInteractiveView extends ViewPart implements
 			}
 		});
 
-		TreeViewerColumn commitIDColumn = createColumn(headings[2], 70);
+		TreeViewerColumn commitIDColumn = createColumn(headings[3], 70);
 		commitIDColumn.setLabelProvider(new HighlightingColumnLabelProvider() {
 			@Override
 			public String getText(Object element) {
@@ -544,7 +723,7 @@ public class RebaseInteractiveView extends ViewPart implements
 			}
 		});
 
-		TreeViewerColumn commitMessageColumn = createColumn(headings[3], 200);
+		TreeViewerColumn commitMessageColumn = createColumn(headings[4], 200);
 		commitMessageColumn
 				.setLabelProvider(new HighlightingColumnLabelProvider() {
 					@Override
@@ -557,7 +736,7 @@ public class RebaseInteractiveView extends ViewPart implements
 					}
 				});
 
-		TreeViewerColumn authorColumn = createColumn(headings[4], 120);
+		TreeViewerColumn authorColumn = createColumn(headings[5], 120);
 		authorColumn.setLabelProvider(new HighlightingColumnLabelProvider() {
 			@Override
 			public String getText(Object element) {
@@ -569,7 +748,7 @@ public class RebaseInteractiveView extends ViewPart implements
 			}
 		});
 
-		TreeViewerColumn authoredDateColumn = createColumn(headings[5], 80);
+		TreeViewerColumn authoredDateColumn = createColumn(headings[6], 80);
 		authoredDateColumn
 				.setLabelProvider(new HighlightingColumnLabelProvider() {
 					@Override
@@ -582,7 +761,7 @@ public class RebaseInteractiveView extends ViewPart implements
 					}
 				});
 
-		TreeViewerColumn committerColumn = createColumn(headings[6], 120);
+		TreeViewerColumn committerColumn = createColumn(headings[7], 120);
 		committerColumn.setLabelProvider(new HighlightingColumnLabelProvider() {
 			@Override
 			public String getText(Object element) {
@@ -594,7 +773,7 @@ public class RebaseInteractiveView extends ViewPart implements
 			}
 		});
 
-		TreeViewerColumn commitDateColumn = createColumn(headings[7], 80);
+		TreeViewerColumn commitDateColumn = createColumn(headings[8], 80);
 		commitDateColumn
 				.setLabelProvider(new HighlightingColumnLabelProvider() {
 					@Override
@@ -637,24 +816,46 @@ public class RebaseInteractiveView extends ViewPart implements
 	private void showRepository(final Repository repository) {
 		if (form.isDisposed())
 			return;
-		if (repository == null) {
-			form.setText(UIText.RebaseInteractiveView_NoSelection);
-			return;
-		}
 
-		currentPlan = RebaseInteractivePlan.getPlan(repository);
-		currentPlan.addRebaseInteractivePlanChangeListener(this);
-		form.setText(getRepositoryName(repository));
+		if (currentPlan != null)
+			currentPlan.removeRebaseInteractivePlanChangeListener(this);
+
+		if (planIndexer != null)
+			planIndexer.dispose();
+
+		if (isValidRepo(repository)) {
+			currentPlan = RebaseInteractivePlan.getPlan(repository);
+			planIndexer = new RebasePlanIndexer(currentPlan);
+			currentPlan.addRebaseInteractivePlanChangeListener(this);
+			form.setText(getRepositoryName(repository));
+		} else {
+			currentPlan = null;
+			planIndexer = null;
+			form.setText(UIText.RebaseInteractiveView_NoSelection);
+		}
 		refresh();
 	}
 
+	private boolean isValidRepo(final Repository repository) {
+		return repository != null && !repository.isBare()
+				&& repository.getWorkTree().exists();
+	}
+
 	void refresh() {
 		if (!isReady())
 			return;
 		asyncExec(new Runnable() {
 			public void run() {
-				planTreeViewer.setInput(currentPlan);
-				refreshUI();
+				Tree t = planTreeViewer.getTree();
+				if (t.isDisposed())
+					return;
+				t.setRedraw(false);
+				try {
+					planTreeViewer.setInput(currentPlan);
+					refreshUI();
+				} finally {
+					t.setRedraw(true);
+				}
 			}
 		});
 
@@ -695,7 +896,9 @@ public class RebaseInteractiveView extends ViewPart implements
 		actionToolBarProvider.mapActionItemsToSelection(planTreeViewer
 				.getSelection());
 		if (!currentPlan.hasRebaseBeenStartedYet()) {
-			actionToolBarProvider.getTheToolbar().setEnabled(true);
+			if (!planTreeViewer.getSelection().isEmpty())
+				actionToolBarProvider.getTheToolbar().setEnabled(true);
+
 			startItem.setEnabled(true);
 			abortItem.setEnabled(true);
 			dndEnabled = true;
@@ -704,10 +907,67 @@ public class RebaseInteractiveView extends ViewPart implements
 			skipItem.setEnabled(true);
 			abortItem.setEnabled(true);
 		}
+
+		if (RebaseInteractivePreferences.isOrderReversed()) {
+			Tree tree = planTreeViewer.getTree();
+			int itemCount = tree.getItemCount();
+			if (itemCount > 0) {
+				TreeItem bottomItem = tree.getItem(itemCount - 1);
+				tree.showItem(bottomItem);
+			}
+		}
 	}
 
-	private void createPopupMenu(TreeViewer planViewer) {
-		// TODO create popup menu
+	private void createPopupMenu(final TreeViewer planViewer) {
+		createContextMenuItems(planViewer);
+
+		MenuManager manager = new MenuManager();
+		manager.addMenuListener(new IMenuListener() {
+			public void menuAboutToShow(IMenuManager menuManager) {
+				boolean selectionNotEmpty = !planViewer.getSelection()
+						.isEmpty();
+				boolean rebaseNotStarted = !currentPlan
+						.hasRebaseBeenStartedYet();
+				boolean menuEnabled = selectionNotEmpty && rebaseNotStarted;
+				for (PlanContextMenuAction item : contextMenuItems)
+					item.setEnabled(menuEnabled);
+			}
+		});
+
+		for (PlanContextMenuAction item : contextMenuItems)
+			manager.add(item);
+
+		Menu menu = manager.createContextMenu(planViewer.getControl());
+		planViewer.getControl().setMenu(menu);
+	}
+
+	private void createContextMenuItems(final TreeViewer planViewer) {
+		contextMenuItems = new ArrayList<PlanContextMenuAction>();
+
+		contextMenuItems.add(new PlanContextMenuAction(
+				UIText.RebaseInteractiveStepActionToolBarProvider_PickText,
+				UIIcons.CHERRY_PICK, RebaseInteractivePlan.ElementAction.PICK,
+				planViewer, actionToolBarProvider));
+		contextMenuItems.add(new PlanContextMenuAction(
+				UIText.RebaseInteractiveStepActionToolBarProvider_SkipText,
+				UIIcons.REBASE_SKIP, RebaseInteractivePlan.ElementAction.SKIP,
+				planViewer, actionToolBarProvider));
+		contextMenuItems.add(new PlanContextMenuAction(
+				UIText.RebaseInteractiveStepActionToolBarProvider_EditText,
+				UIIcons.EDITCONFIG, RebaseInteractivePlan.ElementAction.EDIT,
+				planViewer, actionToolBarProvider));
+		contextMenuItems.add(new PlanContextMenuAction(
+				UIText.RebaseInteractiveStepActionToolBarProvider_SquashText,
+				UIIcons.SQUASH_UP, RebaseInteractivePlan.ElementAction.SQUASH,
+				planViewer, actionToolBarProvider));
+		contextMenuItems.add(new PlanContextMenuAction(
+				UIText.RebaseInteractiveStepActionToolBarProvider_FixupText,
+				UIIcons.FIXUP_UP, RebaseInteractivePlan.ElementAction.FIXUP,
+				planViewer, actionToolBarProvider));
+		contextMenuItems.add(new PlanContextMenuAction(
+				UIText.RebaseInteractiveStepActionToolBarProvider_RewordText,
+				UIIcons.REWORD, RebaseInteractivePlan.ElementAction.REWORD,
+				planViewer, actionToolBarProvider));
 	}
 
 	private static GitDateFormatter getNewDateFormatter() {
@@ -743,4 +1003,4 @@ public class RebaseInteractiveView extends ViewPart implements
 			int oldIndex, int newIndex) {
 		planTreeViewer.refresh(true);
 	}
-}
\ No newline at end of file
+}
