@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2011, 2013 Bernard Leach <leachbj@bouncycastle.org> and others.
+ * Copyright (C) 2011, 2014 Bernard Leach <leachbj@bouncycastle.org> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -14,13 +14,14 @@ import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.CONFLICTIN
 import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.MISSING;
 import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.MISSING_AND_CHANGED;
 import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.MODIFIED;
-import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.PARTIALLY_MODIFIED;
+import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.MODIFIED_AND_ADDED;
+import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.MODIFIED_AND_CHANGED;
 import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.REMOVED;
 import static org.eclipse.egit.ui.internal.staging.StagingEntry.State.UNTRACKED;
 
-import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
@@ -33,13 +34,10 @@ import java.util.TreeSet;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.internal.indexdiff.IndexDiffData;
-import org.eclipse.egit.ui.Activator;
-import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.staging.StagingView.Presentation;
 import org.eclipse.egit.ui.internal.staging.StagingView.StagingViewUpdate;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jgit.lib.Repository;
-import org.eclipse.jgit.submodule.SubmoduleWalk;
 import org.eclipse.ui.model.WorkbenchContentProvider;
 
 /**
@@ -60,9 +58,12 @@ public class StagingViewContentProvider extends WorkbenchContentProvider {
 
 	private Repository repository;
 
+	private final EntryComparator comparator;
+
 	StagingViewContentProvider(StagingView stagingView, boolean unstagedSection) {
 		this.stagingView = stagingView;
 		this.unstagedSection = unstagedSection;
+		comparator = new EntryComparator();
 	}
 
 	public Object getParent(Object element) {
@@ -193,12 +194,12 @@ public class StagingViewContentProvider extends WorkbenchContentProvider {
 					else if (child instanceof StagingFolderEntry)
 						((StagingFolderEntry) child).setParent(folderEntry);
 				}
-				Collections.sort(children, EntryComparator.INSTANCE);
+				Collections.sort(children, comparator);
 				folderEntry.setChildren(children.toArray());
 			}
 		}
 
-		Collections.sort(roots, EntryComparator.INSTANCE);
+		Collections.sort(roots, comparator);
 		return roots.toArray();
 	}
 
@@ -305,7 +306,10 @@ public class StagingViewContentProvider extends WorkbenchContentProvider {
 					nodes.add(new StagingEntry(repository, MISSING, file));
 			for (String file : indexDiff.getModified())
 				if (indexDiff.getChanged().contains(file))
-					nodes.add(new StagingEntry(repository, PARTIALLY_MODIFIED,
+					nodes.add(new StagingEntry(repository, MODIFIED_AND_CHANGED,
+							file));
+				else if (indexDiff.getAdded().contains(file))
+					nodes.add(new StagingEntry(repository, MODIFIED_AND_ADDED,
 							file));
 				else
 					nodes.add(new StagingEntry(repository, MODIFIED, file));
@@ -322,16 +326,11 @@ public class StagingViewContentProvider extends WorkbenchContentProvider {
 				nodes.add(new StagingEntry(repository, REMOVED, file));
 		}
 
-		try {
-		SubmoduleWalk walk = SubmoduleWalk.forIndex(repository);
-		while(walk.next())
-			for (StagingEntry entry : nodes)
-				entry.setSubmodule(entry.getPath().equals(walk.getPath()));
-		} catch(IOException e) {
-			Activator.error(UIText.StagingViewContentProvider_SubmoduleError, e);
-		}
+		setSymlinkFileMode(indexDiff, nodes);
+		setSubmoduleFileMode(indexDiff, nodes);
 
 		content = nodes.toArray(new StagingEntry[nodes.size()]);
+		Arrays.sort(content, comparator);
 
 		treeRoots = null;
 		compactTreeRoots = null;
@@ -351,14 +350,36 @@ public class StagingViewContentProvider extends WorkbenchContentProvider {
 			return content.length;
 	}
 
+	/**
+	 * Set file name mode to be enabled or disabled, to keep proper sorting
+	 * order. This mode displays the names of the file first followed by the
+	 * path to the folder that the file is in.
+	 *
+	 * @param enable
+	 */
+	void setFileNameMode(boolean enable) {
+		comparator.fileNameMode = enable;
+		if (content != null) {
+			Arrays.sort(content, comparator);
+		}
+	}
+
 	private static class EntryComparator implements Comparator<Object> {
-		public static EntryComparator INSTANCE = new EntryComparator();
+		boolean fileNameMode;
 
 		public int compare(Object o1, Object o2) {
 			if (o1 instanceof StagingEntry) {
 				if (o2 instanceof StagingEntry) {
 					StagingEntry e1 = (StagingEntry) o1;
 					StagingEntry e2 = (StagingEntry) o2;
+					if (fileNameMode) {
+						int result = e1.getName().compareTo(e2.getName());
+						if (result != 0)
+							return result;
+						else
+							return e1.getParentPath().toString()
+									.compareTo(e2.getParentPath().toString());
+					}
 					return e1.getPath().compareTo(e2.getPath());
 				} else {
 					// Files should come after folders
@@ -380,4 +401,38 @@ public class StagingViewContentProvider extends WorkbenchContentProvider {
 		}
 	}
 
-}
\ No newline at end of file
+	/**
+	 * Set the symlink file mode of the given StagingEntries.
+	 *
+	 * @param indexDiff
+	 *            the index diff
+	 * @param entries
+	 *            the given StagingEntries
+	 */
+	private void setSymlinkFileMode(IndexDiffData indexDiff,
+			Collection<StagingEntry> entries) {
+		final Set<String> symlinks = indexDiff.getSymlinks();
+		for (StagingEntry stagingEntry : entries) {
+			if (symlinks.contains(stagingEntry.getPath()))
+				stagingEntry.setSymlink(true);
+		}
+	}
+
+	/**
+	 * Set the submodule file mode (equivalent to FileMode.GITLINK) of the given
+	 * StagingEntries.
+	 *
+	 * @param indexDiff
+	 *            the index diff
+	 * @param entries
+	 *            the given StagingEntries
+	 */
+	private void setSubmoduleFileMode(IndexDiffData indexDiff,
+			Collection<StagingEntry> entries) {
+		final Set<String> submodules = indexDiff.getSubmodules();
+		for (StagingEntry stagingEntry : entries) {
+			if (submodules.contains(stagingEntry.getPath()))
+				stagingEntry.setSubmodule(true);
+		}
+	}
+}
