@@ -22,13 +22,21 @@ import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.JobFamilies;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.UIText;
-import org.eclipse.jface.dialogs.InputDialog;
+import org.eclipse.jface.dialogs.Dialog;
+import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.window.Window;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revwalk.RevCommit;
+import org.eclipse.swt.SWT;
+import org.eclipse.swt.layout.GridData;
+import org.eclipse.swt.widgets.Button;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.widgets.Text;
 import org.eclipse.ui.PlatformUI;
 
 /**
@@ -52,17 +60,17 @@ public class StashCreateUI {
 	public boolean createStash(Shell shell) {
 		if (!UIUtils.saveAllEditors(repo))
 			return false;
-		InputDialog commitMessageDialog = new InputDialog(shell,
-				UIText.StashCreateCommand_titleEnterCommitMessage,
-				UIText.StashCreateCommand_messageEnterCommitMessage,
-				null, null);
+		StashCreateDialog commitMessageDialog = new StashCreateDialog(shell);
 		if (commitMessageDialog.open() != Window.OK)
 			return false;
 		String message = commitMessageDialog.getValue();
 		if (message.length() == 0)
 			message = null;
 
-		final StashCreateOperation op = new StashCreateOperation(repo, message);
+		boolean includeUntracked = commitMessageDialog.getIncludeUntracked();
+
+		final StashCreateOperation op = new StashCreateOperation(repo, message,
+				includeUntracked);
 		Job job = new WorkspaceJob(UIText.StashCreateCommand_jobTitle) {
 
 			@Override
@@ -108,4 +116,77 @@ public class StashCreateUI {
 		});
 	}
 
+	private static class StashCreateDialog extends Dialog {
+
+		/**
+		 * Commit message widget.
+		 */
+		private Text text;
+
+		/**
+		 * Include untracked checkbox.
+		 */
+		private Button untrackedButton;
+
+		/**
+		 * The input value; the empty string by default.
+		 */
+		private String commitMessage = ""; //$NON-NLS-1$
+
+		private boolean includeUntracked;
+
+		public StashCreateDialog(Shell shell) {
+			super(shell);
+		}
+
+		@Override
+		protected Control createDialogArea(Composite parent) {
+			Composite composite = (Composite) super.createDialogArea(parent);
+
+			getShell().setText(
+					UIText.StashCreateCommand_titleEnterCommitMessage);
+
+			Label label = new Label(composite, SWT.WRAP);
+			label.setText(UIText.StashCreateCommand_messageEnterCommitMessage);
+			GridData data = new GridData(GridData.GRAB_HORIZONTAL
+					| GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
+					| GridData.VERTICAL_ALIGN_CENTER);
+			data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
+			label.setLayoutData(data);
+			label.setFont(parent.getFont());
+
+			text = new Text(composite, SWT.SINGLE | SWT.BORDER);
+			text.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
+					| GridData.HORIZONTAL_ALIGN_FILL));
+
+			untrackedButton = new Button(composite, SWT.CHECK);
+			untrackedButton
+					.setText(UIText.StashCreateCommand_includeUntrackedLabel);
+
+			text.setFocus();
+			return composite;
+		}
+
+		protected void buttonPressed(int buttonId) {
+			if (buttonId == IDialogConstants.OK_ID) {
+				commitMessage = text.getText();
+				includeUntracked = untrackedButton.getSelection();
+			} else {
+				commitMessage = null;
+				includeUntracked = false;
+			}
+			super.buttonPressed(buttonId);
+		}
+
+		public String getValue() {
+			return commitMessage;
+		}
+
+		public boolean getIncludeUntracked() {
+			return includeUntracked;
+		}
+
+
+	}
+
 }
